#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2017 F5 Networks Inc.
# Copyright (c) 2013 Matt Hite <mhite@hotmail.com>
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: bigip_facts
short_description: Collect facts from F5 BIG-IP devices
description:
  - Collect facts from F5 BIG-IP devices via iControl SOAP API
version_added: 1.6
author:
  - Matt Hite (@mhite)
  - Tim Rupp (@caphrim007)
notes:
  - Requires BIG-IP software version >= 11.4
  - F5 developed module 'bigsuds' required (see http://devcentral.f5.com)
  - Best run as a local_action in your playbook
  - Tested with manager and above account privilege level
  - C(provision) facts were added in 2.2
  - This module is deprecated. Use the C(bigip_device_facts) module instead.
requirements:
  - bigsuds
options:
  session:
    description:
      - BIG-IP session support; may be useful to avoid concurrency
        issues in certain circumstances.
    default: no
    type: bool
  include:
    description:
      - Fact category or list of categories to collect
    required: True
    choices:
      - address_class
      - certificate
      - client_ssl_profile
      - device
      - device_group
      - interface
      - key
      - node
      - pool
      - provision
      - rule
      - self_ip
      - software
      - system_info
      - traffic_group
      - trunk
      - virtual_address
      - virtual_server
      - vlan
  filter:
    description:
      - Shell-style glob matching string used to filter fact keys. Not
        applicable for software, provision, and system_info fact categories.
extends_documentation_fragment: f5
'''

EXAMPLES = r'''
- name: Collect BIG-IP facts
  bigip_facts:
    server: lb.mydomain.com
    user: admin
    password: secret
    include:
      - interface
      - vlan
  delegate_to: localhost
'''

import fnmatch
import re
import traceback

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.six import string_types
from ansible.module_utils.six.moves import map, zip

try:
    from library.module_utils.network.f5.legacy import bigip_api, bigsuds_found
    from library.module_utils.network.f5.common import f5_argument_spec
except ImportError:
    from ansible.module_utils.network.f5.legacy import bigip_api, bigsuds_found
    from ansible.module_utils.network.f5.common import f5_argument_spec

try:
    from suds import MethodNotFound, WebFault
except ImportError:
    pass  # Handle via f5_utils.bigsuds_found


class F5(object):
    """F5 iControl class.

    F5 BIG-IP iControl API class.

    Attributes:
        api: iControl API instance.
    """

    def __init__(self, host, user, password, session=False, validate_certs=True, port=443):
        self.api = bigip_api(host, user, password, validate_certs, port)
        if session:
            self.start_session()

    def start_session(self):
        self.api = self.api.with_session_id()

    def get_api(self):
        return self.api

    def set_recursive_query_state(self, state):
        self.api.System.Session.set_recursive_query_state(state)

    def get_recursive_query_state(self):
        return self.api.System.Session.get_recursive_query_state()

    def enable_recursive_query_state(self):
        self.set_recursive_query_state('STATE_ENABLED')

    def disable_recursive_query_state(self):
        self.set_recursive_query_state('STATE_DISABLED')

    def set_active_folder(self, folder):
        self.api.System.Session.set_active_folder(folder=folder)

    def get_active_folder(self):
        return self.api.System.Session.get_active_folder()


class Interfaces(object):
    """Interfaces class.

    F5 BIG-IP interfaces class.

    Attributes:
        api: iControl API instance.
        interfaces: A list of BIG-IP interface names.
    """

    def __init__(self, api, regex=None):
        self.api = api
        self.interfaces = api.Networking.Interfaces.get_list()
        if regex:
            re_filter = re.compile(regex)
            self.interfaces = filter(re_filter.search, self.interfaces)

    def get_list(self):
        return self.interfaces

    def get_active_media(self):
        return self.api.Networking.Interfaces.get_active_media(self.interfaces)

    def get_actual_flow_control(self):
        return self.api.Networking.Interfaces.get_actual_flow_control(self.interfaces)

    def get_bundle_state(self):
        return self.api.Networking.Interfaces.get_bundle_state(self.interfaces)

    def get_description(self):
        return self.api.Networking.Interfaces.get_description(self.interfaces)

    def get_dual_media_state(self):
        return self.api.Networking.Interfaces.get_dual_media_state(self.interfaces)

    def get_enabled_state(self):
        return self.api.Networking.Interfaces.get_enabled_state(self.interfaces)

    def get_if_index(self):
        return self.api.Networking.Interfaces.get_if_index(self.interfaces)

    def get_learning_mode(self):
        return self.api.Networking.Interfaces.get_learning_mode(self.interfaces)

    def get_lldp_admin_status(self):
        return self.api.Networking.Interfaces.get_lldp_admin_status(self.interfaces)

    def get_lldp_tlvmap(self):
        return self.api.Networking.Interfaces.get_lldp_tlvmap(self.interfaces)

    def get_mac_address(self):
        return self.api.Networking.Interfaces.get_mac_address(self.interfaces)

    def get_media(self):
        return self.api.Networking.Interfaces.get_media(self.interfaces)

    def get_media_option(self):
        return self.api.Networking.Interfaces.get_media_option(self.interfaces)

    def get_media_option_sfp(self):
        return self.api.Networking.Interfaces.get_media_option_sfp(self.interfaces)

    def get_media_sfp(self):
        return self.api.Networking.Interfaces.get_media_sfp(self.interfaces)

    def get_media_speed(self):
        return self.api.Networking.Interfaces.get_media_speed(self.interfaces)

    def get_media_status(self):
        return self.api.Networking.Interfaces.get_media_status(self.interfaces)

    def get_mtu(self):
        return self.api.Networking.Interfaces.get_mtu(self.interfaces)

    def get_phy_master_slave_mode(self):
        return self.api.Networking.Interfaces.get_phy_master_slave_mode(self.interfaces)

    def get_prefer_sfp_state(self):
        return self.api.Networking.Interfaces.get_prefer_sfp_state(self.interfaces)

    def get_flow_control(self):
        return self.api.Networking.Interfaces.get_requested_flow_control(self.interfaces)

    def get_sflow_poll_interval(self):
        return self.api.Networking.Interfaces.get_sflow_poll_interval(self.interfaces)

    def get_sflow_poll_interval_global(self):
        return self.api.Networking.Interfaces.get_sflow_poll_interval_global(self.interfaces)

    def get_sfp_media_state(self):
        return self.api.Networking.Interfaces.get_sfp_media_state(self.interfaces)

    def get_stp_active_edge_port_state(self):
        return self.api.Networking.Interfaces.get_stp_active_edge_port_state(self.interfaces)

    def get_stp_enabled_state(self):
        return self.api.Networking.Interfaces.get_stp_enabled_state(self.interfaces)

    def get_stp_link_type(self):
        return self.api.Networking.Interfaces.get_stp_link_type(self.interfaces)

    def get_stp_protocol_detection_reset_state(self):
        return self.api.Networking.Interfaces.get_stp_protocol_detection_reset_state(self.interfaces)


class SelfIPs(object):
    """Self IPs class.

    F5 BIG-IP Self IPs class.

    Attributes:
        api: iControl API instance.
        self_ips: List of self IPs.
    """

    def __init__(self, api, regex=None):
        self.api = api
        self.self_ips = api.Networking.SelfIPV2.get_list()
        if regex:
            re_filter = re.compile(regex)
            self.self_ips = filter(re_filter.search, self.self_ips)

    def get_list(self):
        return self.self_ips

    def get_address(self):
        return self.api.Networking.SelfIPV2.get_address(self.self_ips)

    def get_allow_access_list(self):
        return self.api.Networking.SelfIPV2.get_allow_access_list(self.self_ips)

    def get_description(self):
        return self.api.Networking.SelfIPV2.get_description(self.self_ips)

    def get_enforced_firewall_policy(self):
        return self.api.Networking.SelfIPV2.get_enforced_firewall_policy(self.self_ips)

    def get_floating_state(self):
        return self.api.Networking.SelfIPV2.get_floating_state(self.self_ips)

    def get_fw_rule(self):
        return self.api.Networking.SelfIPV2.get_fw_rule(self.self_ips)

    def get_netmask(self):
        return self.api.Networking.SelfIPV2.get_netmask(self.self_ips)

    def get_staged_firewall_policy(self):
        return self.api.Networking.SelfIPV2.get_staged_firewall_policy(self.self_ips)

    def get_traffic_group(self):
        return self.api.Networking.SelfIPV2.get_traffic_group(self.self_ips)

    def get_vlan(self):
        return self.api.Networking.SelfIPV2.get_vlan(self.self_ips)

    def get_is_traffic_group_inherited(self):
        return self.api.Networking.SelfIPV2.is_traffic_group_inherited(self.self_ips)


class Trunks(object):
    """Trunks class.

    F5 BIG-IP trunks class.

    Attributes:
        api: iControl API instance.
        trunks: List of trunks.
    """

    def __init__(self, api, regex=None):
        self.api = api
        self.trunks = api.Networking.Trunk.get_list()
        if regex:
            re_filter = re.compile(regex)
            self.trunks = filter(re_filter.search, self.trunks)

    def get_list(self):
        return self.trunks

    def get_active_lacp_state(self):
        return self.api.Networking.Trunk.get_active_lacp_state(self.trunks)

    def get_configured_member_count(self):
        return self.api.Networking.Trunk.get_configured_member_count(self.trunks)

    def get_description(self):
        return self.api.Networking.Trunk.get_description(self.trunks)

    def get_distribution_hash_option(self):
        return self.api.Networking.Trunk.get_distribution_hash_option(self.trunks)

    def get_interface(self):
        return self.api.Networking.Trunk.get_interface(self.trunks)

    def get_lacp_enabled_state(self):
        return self.api.Networking.Trunk.get_lacp_enabled_state(self.trunks)

    def get_lacp_timeout_option(self):
        return self.api.Networking.Trunk.get_lacp_timeout_option(self.trunks)

    def get_link_selection_policy(self):
        return self.api.Networking.Trunk.get_link_selection_policy(self.trunks)

    def get_media_speed(self):
        return self.api.Networking.Trunk.get_media_speed(self.trunks)

    def get_media_status(self):
        return self.api.Networking.Trunk.get_media_status(self.trunks)

    def get_operational_member_count(self):
        return self.api.Networking.Trunk.get_operational_member_count(self.trunks)

    def get_stp_enabled_state(self):
        return self.api.Networking.Trunk.get_stp_enabled_state(self.trunks)

    def get_stp_protocol_detection_reset_state(self):
        return self.api.Networking.Trunk.get_stp_protocol_detection_reset_state(self.trunks)


class Vlans(object):
    """Vlans class.

    F5 BIG-IP Vlans class.

    Attributes:
        api: iControl API instance.
        vlans: List of VLANs.
    """

    def __init__(self, api, regex=None):
        self.api = api
        self.vlans = api.Networking.VLAN.get_list()
        if regex:
            re_filter = re.compile(regex)
            self.vlans = filter(re_filter.search, self.vlans)

    def get_list(self):
        return self.vlans

    def get_auto_lasthop(self):
        return self.api.Networking.VLAN.get_auto_lasthop(self.vlans)

    def get_cmp_hash_algorithm(self):
        return self.api.Networking.VLAN.get_cmp_hash_algorithm(self.vlans)

    def get_description(self):
        return self.api.Networking.VLAN.get_description(self.vlans)

    def get_dynamic_forwarding(self):
        return self.api.Networking.VLAN.get_dynamic_forwarding(self.vlans)

    def get_failsafe_action(self):
        return self.api.Networking.VLAN.get_failsafe_action(self.vlans)

    def get_failsafe_state(self):
        return self.api.Networking.VLAN.get_failsafe_state(self.vlans)

    def get_failsafe_timeout(self):
        return self.api.Networking.VLAN.get_failsafe_timeout(self.vlans)

    def get_if_index(self):
        return self.api.Networking.VLAN.get_if_index(self.vlans)

    def get_learning_mode(self):
        return self.api.Networking.VLAN.get_learning_mode(self.vlans)

    def get_mac_masquerade_address(self):
        return self.api.Networking.VLAN.get_mac_masquerade_address(self.vlans)

    def get_member(self):
        return self.api.Networking.VLAN.get_member(self.vlans)

    def get_mtu(self):
        return self.api.Networking.VLAN.get_mtu(self.vlans)

    def get_sflow_poll_interval(self):
        return self.api.Networking.VLAN.get_sflow_poll_interval(self.vlans)

    def get_sflow_poll_interval_global(self):
        return self.api.Networking.VLAN.get_sflow_poll_interval_global(self.vlans)

    def get_sflow_sampling_rate(self):
        return self.api.Networking.VLAN.get_sflow_sampling_rate(self.vlans)

    def get_sflow_sampling_rate_global(self):
        return self.api.Networking.VLAN.get_sflow_sampling_rate_global(self.vlans)

    def get_source_check_state(self):
        return self.api.Networking.VLAN.get_source_check_state(self.vlans)

    def get_true_mac_address(self):
        return self.api.Networking.VLAN.get_true_mac_address(self.vlans)

    def get_vlan_id(self):
        return self.api.Networking.VLAN.get_vlan_id(self.vlans)


class Software(object):
    """Software class.

    F5 BIG-IP software class.

    Attributes:
        api: iControl API instance.
    """

    def __init__(self, api):
        self.api = api

    def get_all_software_status(self):
        return self.api.System.SoftwareManagement.get_all_software_status()


class VirtualServers(object):
    """Virtual servers class.

    F5 BIG-IP virtual servers class.

    Attributes:
        api: iControl API instance.
        virtual_servers: List of virtual servers.
    """

    def __init__(self, api, regex=None):
        self.api = api
        self.virtual_servers = api.LocalLB.VirtualServer.get_list()
        if regex:
            re_filter = re.compile(regex)
            self.virtual_servers = filter(re_filter.search, self.virtual_servers)

    def get_list(self):
        return self.virtual_servers

    def get_name(self):
        return [x[x.rfind('/') + 1:] for x in self.virtual_servers]

    def get_actual_hardware_acceleration(self):
        return self.api.LocalLB.VirtualServer.get_actual_hardware_acceleration(self.virtual_servers)

    def get_authentication_profile(self):
        return self.api.LocalLB.VirtualServer.get_authentication_profile(self.virtual_servers)

    def get_auto_lasthop(self):
        return self.api.LocalLB.VirtualServer.get_auto_lasthop(self.virtual_servers)

    def get_bw_controller_policy(self):
        return self.api.LocalLB.VirtualServer.get_bw_controller_policy(self.virtual_servers)

    def get_clone_pool(self):
        return self.api.LocalLB.VirtualServer.get_clone_pool(self.virtual_servers)

    def get_cmp_enable_mode(self):
        return self.api.LocalLB.VirtualServer.get_cmp_enable_mode(self.virtual_servers)

    def get_connection_limit(self):
        return self.api.LocalLB.VirtualServer.get_connection_limit(self.virtual_servers)

    def get_connection_mirror_state(self):
        return self.api.LocalLB.VirtualServer.get_connection_mirror_state(self.virtual_servers)

    def get_default_pool_name(self):
        return self.api.LocalLB.VirtualServer.get_default_pool_name(self.virtual_servers)

    def get_description(self):
        return self.api.LocalLB.VirtualServer.get_description(self.virtual_servers)

    def get_destination(self):
        return self.api.LocalLB.VirtualServer.get_destination_v2(self.virtual_servers)

    def get_enabled_state(self):
        return self.api.LocalLB.VirtualServer.get_enabled_state(self.virtual_servers)

    def get_enforced_firewall_policy(self):
        return self.api.LocalLB.VirtualServer.get_enforced_firewall_policy(self.virtual_servers)

    def get_fallback_persistence_profile(self):
        return self.api.LocalLB.VirtualServer.get_fallback_persistence_profile(self.virtual_servers)

    def get_fw_rule(self):
        return self.api.LocalLB.VirtualServer.get_fw_rule(self.virtual_servers)

    def get_gtm_score(self):
        return self.api.LocalLB.VirtualServer.get_gtm_score(self.virtual_servers)

    def get_last_hop_pool(self):
        return self.api.LocalLB.VirtualServer.get_last_hop_pool(self.virtual_servers)

    def get_nat64_state(self):
        return self.api.LocalLB.VirtualServer.get_nat64_state(self.virtual_servers)

    def get_object_status(self):
        return self.api.LocalLB.VirtualServer.get_object_status(self.virtual_servers)

    def get_persistence_profile(self):
        return self.api.LocalLB.VirtualServer.get_persistence_profile(self.virtual_servers)

    def get_profile(self):
        return self.api.LocalLB.VirtualServer.get_profile(self.virtual_servers)

    def get_protocol(self):
        return self.api.LocalLB.VirtualServer.get_protocol(self.virtual_servers)

    def get_rate_class(self):
        return self.api.LocalLB.VirtualServer.get_rate_class(self.virtual_servers)

    def get_rate_limit(self):
        return self.api.LocalLB.VirtualServer.get_rate_limit(self.virtual_servers)

    def get_rate_limit_destination_mask(self):
        return self.api.LocalLB.VirtualServer.get_rate_limit_destination_mask(self.virtual_servers)

    def get_rate_limit_mode(self):
        return self.api.LocalLB.VirtualServer.get_rate_limit_mode(self.virtual_servers)

    def get_rate_limit_source_mask(self):
        return self.api.LocalLB.VirtualServer.get_rate_limit_source_mask(self.virtual_servers)

    def get_related_rule(self):
        return self.api.LocalLB.VirtualServer.get_related_rule(self.virtual_servers)

    def get_rule(self):
        return self.api.LocalLB.VirtualServer.get_rule(self.virtual_servers)

    def get_security_log_profile(self):
        return self.api.LocalLB.VirtualServer.get_security_log_profile(self.virtual_servers)

    def get_snat_pool(self):
        return self.api.LocalLB.VirtualServer.get_snat_pool(self.virtual_servers)

    def get_snat_type(self):
        return self.api.LocalLB.VirtualServer.get_snat_type(self.virtual_servers)

    def get_source_address(self):
        return self.api.LocalLB.VirtualServer.get_source_address(self.virtual_servers)

    def get_source_address_translation_lsn_pool(self):
        return self.api.LocalLB.VirtualServer.get_source_address_translation_lsn_pool(self.virtual_servers)

    def get_source_address_translation_snat_pool(self):
        return self.api.LocalLB.VirtualServer.get_source_address_translation_snat_pool(self.virtual_servers)

    def get_source_address_translation_type(self):
        return self.api.LocalLB.VirtualServer.get_source_address_translation_type(self.virtual_servers)

    def get_source_port_behavior(self):
        return self.api.LocalLB.VirtualServer.get_source_port_behavior(self.virtual_servers)

    def get_staged_firewall_policy(self):
        return self.api.LocalLB.VirtualServer.get_staged_firewall_policy(self.virtual_servers)

    def get_translate_address_state(self):
        return self.api.LocalLB.VirtualServer.get_translate_address_state(self.virtual_servers)

    def get_translate_port_state(self):
        return self.api.LocalLB.VirtualServer.get_translate_port_state(self.virtual_servers)

    def get_type(self):
        return self.api.LocalLB.VirtualServer.get_type(self.virtual_servers)

    def get_vlan(self):
        return self.api.LocalLB.VirtualServer.get_vlan(self.virtual_servers)

    def get_wildmask(self):
        return self.api.LocalLB.VirtualServer.get_wildmask(self.virtual_servers)


class Pools(object):
    """Pools class.

    F5 BIG-IP pools class.

    Attributes:
        api: iControl API instance.
        pool_names: List of pool names.
    """

    def __init__(self, api, regex=None):
        self.api = api
        self.pool_names = api.LocalLB.Pool.get_list()
        if regex:
            re_filter = re.compile(regex)
            self.pool_names = filter(re_filter.search, self.pool_names)

    def get_list(self):
        return self.pool_names

    def get_name(self):
        return [x[x.rfind('/') + 1:] for x in self.pool_names]

    def get_action_on_service_down(self):
        return self.api.LocalLB.Pool.get_action_on_service_down(self.pool_names)

    def get_active_member_count(self):
        return self.api.LocalLB.Pool.get_active_member_count(self.pool_names)

    def get_aggregate_dynamic_ratio(self):
        return self.api.LocalLB.Pool.get_aggregate_dynamic_ratio(self.pool_names)

    def get_allow_nat_state(self):
        return self.api.LocalLB.Pool.get_allow_nat_state(self.pool_names)

    def get_allow_snat_state(self):
        return self.api.LocalLB.Pool.get_allow_snat_state(self.pool_names)

    def get_client_ip_tos(self):
        return self.api.LocalLB.Pool.get_client_ip_tos(self.pool_names)

    def get_client_link_qos(self):
        return self.api.LocalLB.Pool.get_client_link_qos(self.pool_names)

    def get_description(self):
        return self.api.LocalLB.Pool.get_description(self.pool_names)

    def get_gateway_failsafe_device(self):
        return self.api.LocalLB.Pool.get_gateway_failsafe_device(self.pool_names)

    def get_ignore_persisted_weight_state(self):
        return self.api.LocalLB.Pool.get_ignore_persisted_weight_state(self.pool_names)

    def get_lb_method(self):
        result = []
        lb_choice = dict(
            LB_METHOD_DYNAMIC_RATIO_MEMBER='dynamic-ratio-member',
            LB_METHOD_DYNAMIC_RATIO='dynamic-ratio-node',
            LB_METHOD_FASTEST_APP_RESPONSE='fastest-app-response',
            LB_METHOD_FASTEST_NODE_ADDRESS='fastest-node',
            LB_METHOD_LEAST_CONNECTION_MEMBER='least-connections-member',
            LB_METHOD_LEAST_CONNECTION_NODE_ADDRESS='least-connections-node',
            LB_METHOD_LEAST_SESSIONS='least-sessions',
            LB_METHOD_OBSERVED_MEMBER='observed-member',
            LB_METHOD_OBSERVED_NODE_ADDRESS='observed-node',
            LB_METHOD_PREDICTIVE_MEMBER='predictive-member',
            LB_METHOD_PREDICTIVE_NODE_ADDRESS='predictive-node',
            LB_METHOD_RATIO_LEAST_CONNECTION_MEMBER='ratio-least-connections-member',
            LB_METHOD_RATIO_LEAST_CONNECTION_NODE_ADDRESS='ratio-least-connections-node',
            LB_METHOD_RATIO_MEMBER='ratio-member',
            LB_METHOD_RATIO_NODE_ADDRESS='ratio-node',
            LB_METHOD_RATIO_SESSION='ratio-session',
            LB_METHOD_ROUND_ROBIN='round-robin',
            LB_METHOD_WEIGHTED_LEAST_CONNECTION_MEMBER='weighted-least-connections-member',
            LB_METHOD_WEIGHTED_LEAST_CONNECTION_NODE_ADDRESS='weighted-least-connections-node'
        )
        methods = self.api.LocalLB.Pool.get_lb_method(self.pool_names)
        for method in methods:
            result.append(lb_choice.get(method, method))
        return result

    def get_member(self):
        return self.api.LocalLB.Pool.get_member_v2(self.pool_names)

    def get_minimum_active_member(self):
        return self.api.LocalLB.Pool.get_minimum_active_member(self.pool_names)

    def get_minimum_up_member(self):
        return self.api.LocalLB.Pool.get_minimum_up_member(self.pool_names)

    def get_minimum_up_member_action(self):
        return self.api.LocalLB.Pool.get_minimum_up_member_action(self.pool_names)

    def get_minimum_up_member_enabled_state(self):
        return self.api.LocalLB.Pool.get_minimum_up_member_enabled_state(self.pool_names)

    def get_monitor_association(self):
        return self.api.LocalLB.Pool.get_monitor_association(self.pool_names)

    def get_monitor_instance(self):
        return self.api.LocalLB.Pool.get_monitor_instance(self.pool_names)

    def get_object_status(self):
        return self.api.LocalLB.Pool.get_object_status(self.pool_names)

    def get_profile(self):
        return self.api.LocalLB.Pool.get_profile(self.pool_names)

    def get_queue_depth_limit(self):
        return self.api.LocalLB.Pool.get_queue_depth_limit(self.pool_names)

    def get_queue_on_connection_limit_state(self):
        return self.api.LocalLB.Pool.get_queue_on_connection_limit_state(self.pool_names)

    def get_queue_time_limit(self):
        return self.api.LocalLB.Pool.get_queue_time_limit(self.pool_names)

    def get_reselect_tries(self):
        return self.api.LocalLB.Pool.get_reselect_tries(self.pool_names)

    def get_server_ip_tos(self):
        return self.api.LocalLB.Pool.get_server_ip_tos(self.pool_names)

    def get_server_link_qos(self):
        return self.api.LocalLB.Pool.get_server_link_qos(self.pool_names)

    def get_simple_timeout(self):
        return self.api.LocalLB.Pool.get_simple_timeout(self.pool_names)

    def get_slow_ramp_time(self):
        return self.api.LocalLB.Pool.get_slow_ramp_time(self.pool_names)


class Devices(object):
    """Devices class.

    F5 BIG-IP devices class.

    Attributes:
        api: iControl API instance.
        devices: List of devices.
    """

    def __init__(self, api, regex=None):
        self.api = api
        self.devices = api.Management.Device.get_list()
        if regex:
            re_filter = re.compile(regex)
            self.devices = filter(re_filter.search, self.devices)

    def get_list(self):
        return self.devices

    def get_active_modules(self):
        return self.api.Management.Device.get_active_modules(self.devices)

    def get_base_mac_address(self):
        return self.api.Management.Device.get_base_mac_address(self.devices)

    def get_blade_addresses(self):
        return self.api.Management.Device.get_blade_addresses(self.devices)

    def get_build(self):
        return self.api.Management.Device.get_build(self.devices)

    def get_chassis_id(self):
        return self.api.Management.Device.get_chassis_id(self.devices)

    def get_chassis_type(self):
        return self.api.Management.Device.get_chassis_type(self.devices)

    def get_comment(self):
        return self.api.Management.Device.get_comment(self.devices)

    def get_configsync_address(self):
        return self.api.Management.Device.get_configsync_address(self.devices)

    def get_contact(self):
        return self.api.Management.Device.get_contact(self.devices)

    def get_description(self):
        return self.api.Management.Device.get_description(self.devices)

    def get_edition(self):
        return self.api.Management.Device.get_edition(self.devices)

    def get_failover_state(self):
        return self.api.Management.Device.get_failover_state(self.devices)

    def get_local_device(self):
        return self.api.Management.Device.get_local_device()

    def get_hostname(self):
        return self.api.Management.Device.get_hostname(self.devices)

    def get_inactive_modules(self):
        return self.api.Management.Device.get_inactive_modules(self.devices)

    def get_location(self):
        return self.api.Management.Device.get_location(self.devices)

    def get_management_address(self):
        return self.api.Management.Device.get_management_address(self.devices)

    def get_marketing_name(self):
        return self.api.Management.Device.get_marketing_name(self.devices)

    def get_multicast_address(self):
        return self.api.Management.Device.get_multicast_address(self.devices)

    def get_optional_modules(self):
        return self.api.Management.Device.get_optional_modules(self.devices)

    def get_platform_id(self):
        return self.api.Management.Device.get_platform_id(self.devices)

    def get_primary_mirror_address(self):
        return self.api.Management.Device.get_primary_mirror_address(self.devices)

    def get_product(self):
        return self.api.Management.Device.get_product(self.devices)

    def get_secondary_mirror_address(self):
        return self.api.Management.Device.get_secondary_mirror_address(self.devices)

    def get_software_version(self):
        return self.api.Management.Device.get_software_version(self.devices)

    def get_timelimited_modules(self):
        return self.api.Management.Device.get_timelimited_modules(self.devices)

    def get_timezone(self):
        return self.api.Management.Device.get_timezone(self.devices)

    def get_unicast_addresses(self):
        return self.api.Management.Device.get_unicast_addresses(self.devices)


class DeviceGroups(object):
    """Device groups class.

    F5 BIG-IP device groups class.

    Attributes:
        api: iControl API instance.
        device_groups: List of device groups.
    """

    def __init__(self, api, regex=None):
        self.api = api
        self.device_groups = api.Management.DeviceGroup.get_list()
        if regex:
            re_filter = re.compile(regex)
            self.device_groups = filter(re_filter.search, self.device_groups)

    def get_list(self):
        return self.device_groups

    def get_all_preferred_active(self):
        return self.api.Management.DeviceGroup.get_all_preferred_active(self.device_groups)

    def get_autosync_enabled_state(self):
        return self.api.Management.DeviceGroup.get_autosync_enabled_state(self.device_groups)

    def get_description(self):
        return self.api.Management.DeviceGroup.get_description(self.device_groups)

    def get_device(self):
        return self.api.Management.DeviceGroup.get_device(self.device_groups)

    def get_full_load_on_sync_state(self):
        return self.api.Management.DeviceGroup.get_full_load_on_sync_state(self.device_groups)

    def get_incremental_config_sync_size_maximum(self):
        return self.api.Management.DeviceGroup.get_incremental_config_sync_size_maximum(self.device_groups)

    def get_network_failover_enabled_state(self):
        return self.api.Management.DeviceGroup.get_network_failover_enabled_state(self.device_groups)

    def get_sync_status(self):
        return self.api.Management.DeviceGroup.get_sync_status(self.device_groups)

    def get_type(self):
        return self.api.Management.DeviceGroup.get_type(self.device_groups)


class TrafficGroups(object):
    """Traffic groups class.

    F5 BIG-IP traffic groups class.

    Attributes:
        api: iControl API instance.
        traffic_groups: List of traffic groups.
    """

    def __init__(self, api, regex=None):
        self.api = api
        self.traffic_groups = api.Management.TrafficGroup.get_list()
        if regex:
            re_filter = re.compile(regex)
            self.traffic_groups = filter(re_filter.search, self.traffic_groups)

    def get_list(self):
        return self.traffic_groups

    def get_auto_failback_enabled_state(self):
        return self.api.Management.TrafficGroup.get_auto_failback_enabled_state(self.traffic_groups)

    def get_auto_failback_time(self):
        return self.api.Management.TrafficGroup.get_auto_failback_time(self.traffic_groups)

    def get_default_device(self):
        return self.api.Management.TrafficGroup.get_default_device(self.traffic_groups)

    def get_description(self):
        return self.api.Management.TrafficGroup.get_description(self.traffic_groups)

    def get_ha_load_factor(self):
        return self.api.Management.TrafficGroup.get_ha_load_factor(self.traffic_groups)

    def get_ha_order(self):
        return self.api.Management.TrafficGroup.get_ha_order(self.traffic_groups)

    def get_is_floating(self):
        return self.api.Management.TrafficGroup.get_is_floating(self.traffic_groups)

    def get_mac_masquerade_address(self):
        return self.api.Management.TrafficGroup.get_mac_masquerade_address(self.traffic_groups)

    def get_unit_id(self):
        return self.api.Management.TrafficGroup.get_unit_id(self.traffic_groups)


class Rules(object):
    """Rules class.

    F5 BIG-IP iRules class.

    Attributes:
        api: iControl API instance.
        rules: List of iRules.
    """

    def __init__(self, api, regex=None):
        self.api = api
        self.rules = api.LocalLB.Rule.get_list()
        if regex:
            re_filter = re.compile(regex)
            self.traffic_groups = filter(re_filter.search, self.rules)

    def get_list(self):
        return self.rules

    def get_description(self):
        return self.api.LocalLB.Rule.get_description(rule_names=self.rules)

    def get_ignore_vertification(self):
        return self.api.LocalLB.Rule.get_ignore_vertification(rule_names=self.rules)

    def get_verification_status(self):
        return self.api.LocalLB.Rule.get_verification_status_v2(rule_names=self.rules)

    def get_definition(self):
        return [x['rule_definition'] for x in self.api.LocalLB.Rule.query_rule(rule_names=self.rules)]


class Nodes(object):
    """Nodes class.

    F5 BIG-IP nodes class.

    Attributes:
        api: iControl API instance.
        nodes: List of nodes.
    """

    def __init__(self, api, regex=None):
        self.api = api
        self.nodes = api.LocalLB.NodeAddressV2.get_list()
        if regex:
            re_filter = re.compile(regex)
            self.nodes = filter(re_filter.search, self.nodes)

    def get_list(self):
        return self.nodes

    def get_address(self):
        return self.api.LocalLB.NodeAddressV2.get_address(nodes=self.nodes)

    def get_name(self):
        return [x[x.rfind('/') + 1:] for x in self.nodes]

    def get_connection_limit(self):
        return self.api.LocalLB.NodeAddressV2.get_connection_limit(nodes=self.nodes)

    def get_description(self):
        return self.api.LocalLB.NodeAddressV2.get_description(nodes=self.nodes)

    def get_dynamic_ratio(self):
        return self.api.LocalLB.NodeAddressV2.get_dynamic_ratio_v2(nodes=self.nodes)

    def get_monitor_instance(self):
        return self.api.LocalLB.NodeAddressV2.get_monitor_instance(nodes=self.nodes)

    def get_monitor_rule(self):
        return self.api.LocalLB.NodeAddressV2.get_monitor_rule(nodes=self.nodes)

    def get_monitor_status(self):
        return self.api.LocalLB.NodeAddressV2.get_monitor_status(nodes=self.nodes)

    def get_object_status(self):
        return self.api.LocalLB.NodeAddressV2.get_object_status(nodes=self.nodes)

    def get_rate_limit(self):
        return self.api.LocalLB.NodeAddressV2.get_rate_limit(nodes=self.nodes)

    def get_ratio(self):
        return self.api.LocalLB.NodeAddressV2.get_ratio(nodes=self.nodes)

    def get_session_status(self):
        return self.api.LocalLB.NodeAddressV2.get_session_status(nodes=self.nodes)


class VirtualAddresses(object):
    """Virtual addresses class.

    F5 BIG-IP virtual addresses class.

    Attributes:
        api: iControl API instance.
        virtual_addresses: List of virtual addresses.
    """

    def __init__(self, api, regex=None):
        self.api = api
        self.virtual_addresses = api.LocalLB.VirtualAddressV2.get_list()
        if regex:
            re_filter = re.compile(regex)
            self.virtual_addresses = filter(re_filter.search, self.virtual_addresses)

    def get_list(self):
        return self.virtual_addresses

    def get_address(self):
        return self.api.LocalLB.VirtualAddressV2.get_address(self.virtual_addresses)

    def get_arp_state(self):
        return self.api.LocalLB.VirtualAddressV2.get_arp_state(self.virtual_addresses)

    def get_auto_delete_state(self):
        return self.api.LocalLB.VirtualAddressV2.get_auto_delete_state(self.virtual_addresses)

    def get_connection_limit(self):
        return self.api.LocalLB.VirtualAddressV2.get_connection_limit(self.virtual_addresses)

    def get_description(self):
        return self.api.LocalLB.VirtualAddressV2.get_description(self.virtual_addresses)

    def get_enabled_state(self):
        return self.api.LocalLB.VirtualAddressV2.get_enabled_state(self.virtual_addresses)

    def get_icmp_echo_state(self):
        return self.api.LocalLB.VirtualAddressV2.get_icmp_echo_state(self.virtual_addresses)

    def get_is_floating_state(self):
        return self.api.LocalLB.VirtualAddressV2.get_is_floating_state(self.virtual_addresses)

    def get_netmask(self):
        return self.api.LocalLB.VirtualAddressV2.get_netmask(self.virtual_addresses)

    def get_object_status(self):
        return self.api.LocalLB.VirtualAddressV2.get_object_status(self.virtual_addresses)

    def get_route_advertisement_state(self):
        return self.api.LocalLB.VirtualAddressV2.get_route_advertisement_state(self.virtual_addresses)

    def get_traffic_group(self):
        return self.api.LocalLB.VirtualAddressV2.get_traffic_group(self.virtual_addresses)


class AddressClasses(object):
    """Address group/class class.

    F5 BIG-IP address group/class class.

    In TMUI these things are known as Address Data Groups. Examples that ship with the
    box include /Common/aol and /Common/private_net

    Attributes:
        api: iControl API instance.
        address_classes: List of address classes.
    """

    def __init__(self, api, regex=None):
        self.api = api
        self.address_classes = api.LocalLB.Class.get_address_class_list()
        if regex:
            re_filter = re.compile(regex)
            self.address_classes = filter(re_filter.search, self.address_classes)

    def get_list(self):
        return self.address_classes

    def get_address_class(self):
        key = self.api.LocalLB.Class.get_address_class(self.address_classes)
        value = self.api.LocalLB.Class.get_address_class_member_data_value(key)

        result = []
        for idx, v in enumerate(key):
            for idx2, member in enumerate(v['members']):
                dg_value = dict(
                    value=value[idx][idx2]
                )
                dg_value.update(member)
                result.append(dg_value)
        return result

    def get_description(self):
        return self.api.LocalLB.Class.get_description(self.address_classes)


class Certificates(object):
    """Certificates class.

    F5 BIG-IP certificates class.

    Attributes:
        api: iControl API instance.
        certificates: List of certificate identifiers.
        certificate_list: List of certificate information structures.
    """

    def __init__(self, api, regex=None, mode="MANAGEMENT_MODE_DEFAULT"):
        self.api = api
        self.certificate_list = api.Management.KeyCertificate.get_certificate_list(mode=mode)
        self.certificates = [x['certificate']['cert_info']['id'] for x in self.certificate_list]
        if regex:
            re_filter = re.compile(regex)
            self.certificates = filter(re_filter.search, self.certificates)
            self.certificate_list = [x for x in self.certificate_list if x['certificate']['cert_info']['id'] in self.certificates]

    def get_list(self):
        return self.certificates

    def get_certificate_list(self):
        return self.certificate_list


class Keys(object):
    """Keys class.

    F5 BIG-IP keys class.

    Attributes:
        api: iControl API instance.
        keys: List of key identifiers.
        key_list: List of key information structures.
    """

    def __init__(self, api, regex=None, mode="MANAGEMENT_MODE_DEFAULT"):
        self.api = api
        self.key_list = api.Management.KeyCertificate.get_key_list(mode=mode)
        self.keys = [x['key_info']['id'] for x in self.key_list]
        if regex:
            re_filter = re.compile(regex)
            self.keys = filter(re_filter.search, self.keys)
            self.key_list = [x for x in self.key_list if x['key_info']['id'] in self.keys]

    def get_list(self):
        return self.keys

    def get_key_list(self):
        return self.key_list


class ProfileClientSSL(object):
    """Client SSL profiles class.

    F5 BIG-IP client SSL profiles class.

    Attributes:
        api: iControl API instance.
        profiles: List of client SSL profiles.
    """

    def __init__(self, api, regex=None):
        self.api = api
        self.profiles = api.LocalLB.ProfileClientSSL.get_list()
        if regex:
            re_filter = re.compile(regex)
            self.profiles = filter(re_filter.search, self.profiles)

    def get_list(self):
        return self.profiles

    def get_alert_timeout(self):
        return self.api.LocalLB.ProfileClientSSL.get_alert_timeout(self.profiles)

    def get_allow_nonssl_state(self):
        return self.api.LocalLB.ProfileClientSSL.get_allow_nonssl_state(self.profiles)

    def get_authenticate_depth(self):
        return self.api.LocalLB.ProfileClientSSL.get_authenticate_depth(self.profiles)

    def get_authenticate_once_state(self):
        return self.api.LocalLB.ProfileClientSSL.get_authenticate_once_state(self.profiles)

    def get_ca_file(self):
        return self.api.LocalLB.ProfileClientSSL.get_ca_file_v2(self.profiles)

    def get_cache_size(self):
        return self.api.LocalLB.ProfileClientSSL.get_cache_size(self.profiles)

    def get_cache_timeout(self):
        return self.api.LocalLB.ProfileClientSSL.get_cache_timeout(self.profiles)

    def get_certificate_file(self):
        return self.api.LocalLB.ProfileClientSSL.get_certificate_file_v2(self.profiles)

    def get_chain_file(self):
        return self.api.LocalLB.ProfileClientSSL.get_chain_file_v2(self.profiles)

    def get_cipher_list(self):
        return self.api.LocalLB.ProfileClientSSL.get_cipher_list(self.profiles)

    def get_client_certificate_ca_file(self):
        return self.api.LocalLB.ProfileClientSSL.get_client_certificate_ca_file_v2(self.profiles)

    def get_crl_file(self):
        return self.api.LocalLB.ProfileClientSSL.get_crl_file_v2(self.profiles)

    def get_default_profile(self):
        return self.api.LocalLB.ProfileClientSSL.get_default_profile(self.profiles)

    def get_description(self):
        return self.api.LocalLB.ProfileClientSSL.get_description(self.profiles)

    def get_forward_proxy_ca_certificate_file(self):
        return self.api.LocalLB.ProfileClientSSL.get_forward_proxy_ca_certificate_file(self.profiles)

    def get_forward_proxy_ca_key_file(self):
        return self.api.LocalLB.ProfileClientSSL.get_forward_proxy_ca_key_file(self.profiles)

    def get_forward_proxy_ca_passphrase(self):
        return self.api.LocalLB.ProfileClientSSL.get_forward_proxy_ca_passphrase(self.profiles)

    def get_forward_proxy_certificate_extension_include(self):
        return self.api.LocalLB.ProfileClientSSL.get_forward_proxy_certificate_extension_include(self.profiles)

    def get_forward_proxy_certificate_lifespan(self):
        return self.api.LocalLB.ProfileClientSSL.get_forward_proxy_certificate_lifespan(self.profiles)

    def get_forward_proxy_enabled_state(self):
        return self.api.LocalLB.ProfileClientSSL.get_forward_proxy_enabled_state(self.profiles)

    def get_forward_proxy_lookup_by_ipaddr_port_state(self):
        return self.api.LocalLB.ProfileClientSSL.get_forward_proxy_lookup_by_ipaddr_port_state(self.profiles)

    def get_handshake_timeout(self):
        return self.api.LocalLB.ProfileClientSSL.get_handshake_timeout(self.profiles)

    def get_key_file(self):
        return self.api.LocalLB.ProfileClientSSL.get_key_file_v2(self.profiles)

    def get_modssl_emulation_state(self):
        return self.api.LocalLB.ProfileClientSSL.get_modssl_emulation_state(self.profiles)

    def get_passphrase(self):
        return self.api.LocalLB.ProfileClientSSL.get_passphrase(self.profiles)

    def get_peer_certification_mode(self):
        return self.api.LocalLB.ProfileClientSSL.get_peer_certification_mode(self.profiles)

    def get_profile_mode(self):
        return self.api.LocalLB.ProfileClientSSL.get_profile_mode(self.profiles)

    def get_renegotiation_maximum_record_delay(self):
        return self.api.LocalLB.ProfileClientSSL.get_renegotiation_maximum_record_delay(self.profiles)

    def get_renegotiation_period(self):
        return self.api.LocalLB.ProfileClientSSL.get_renegotiation_period(self.profiles)

    def get_renegotiation_state(self):
        return self.api.LocalLB.ProfileClientSSL.get_renegotiation_state(self.profiles)

    def get_renegotiation_throughput(self):
        return self.api.LocalLB.ProfileClientSSL.get_renegotiation_throughput(self.profiles)

    def get_retain_certificate_state(self):
        return self.api.LocalLB.ProfileClientSSL.get_retain_certificate_state(self.profiles)

    def get_secure_renegotiation_mode(self):
        return self.api.LocalLB.ProfileClientSSL.get_secure_renegotiation_mode(self.profiles)

    def get_server_name(self):
        return self.api.LocalLB.ProfileClientSSL.get_server_name(self.profiles)

    def get_session_ticket_state(self):
        return self.api.LocalLB.ProfileClientSSL.get_session_ticket_state(self.profiles)

    def get_sni_default_state(self):
        return self.api.LocalLB.ProfileClientSSL.get_sni_default_state(self.profiles)

    def get_sni_require_state(self):
        return self.api.LocalLB.ProfileClientSSL.get_sni_require_state(self.profiles)

    def get_ssl_option(self):
        return self.api.LocalLB.ProfileClientSSL.get_ssl_option(self.profiles)

    def get_strict_resume_state(self):
        return self.api.LocalLB.ProfileClientSSL.get_strict_resume_state(self.profiles)

    def get_unclean_shutdown_state(self):
        return self.api.LocalLB.ProfileClientSSL.get_unclean_shutdown_state(self.profiles)

    def get_is_base_profile(self):
        return self.api.LocalLB.ProfileClientSSL.is_base_profile(self.profiles)

    def get_is_system_profile(self):
        return self.api.LocalLB.ProfileClientSSL.is_system_profile(self.profiles)


class SystemInfo(object):
    """System information class.

    F5 BIG-IP system information class.

    Attributes:
        api: iControl API instance.
    """

    def __init__(self, api):
        self.api = api

    def get_base_mac_address(self):
        return self.api.System.SystemInfo.get_base_mac_address()

    def get_blade_temperature(self):
        return self.api.System.SystemInfo.get_blade_temperature()

    def get_chassis_slot_information(self):
        return self.api.System.SystemInfo.get_chassis_slot_information()

    def get_globally_unique_identifier(self):
        return self.api.System.SystemInfo.get_globally_unique_identifier()

    def get_group_id(self):
        return self.api.System.SystemInfo.get_group_id()

    def get_hardware_information(self):
        return self.api.System.SystemInfo.get_hardware_information()

    def get_marketing_name(self):
        return self.api.System.SystemInfo.get_marketing_name()

    def get_product_information(self):
        return self.api.System.SystemInfo.get_product_information()

    def get_pva_version(self):
        return self.api.System.SystemInfo.get_pva_version()

    def get_system_id(self):
        return self.api.System.SystemInfo.get_system_id()

    def get_system_information(self):
        return self.api.System.SystemInfo.get_system_information()

    def get_time(self):
        return self.api.System.SystemInfo.get_time()

    def get_time_zone(self):
        return self.api.System.SystemInfo.get_time_zone()

    def get_uptime(self):
        return self.api.System.SystemInfo.get_uptime()


class ProvisionInfo(object):
    """Provision information class.

    F5 BIG-IP provision information class.

    Attributes:
        api: iControl API instance.
    """

    def __init__(self, api):
        self.api = api

    def get_list(self):
        result = []
        list = self.api.Management.Provision.get_list()
        for item in list:
            item = item.lower().replace('tmos_module_', '')
            result.append(item)
        return result

    def get_provisioned_list(self):
        result = []
        list = self.api.Management.Provision.get_provisioned_list()
        for item in list:
            item = item.lower().replace('tmos_module_', '')
            result.append(item)
        return result


def generate_dict(api_obj, fields):
    result_dict = {}
    lists = []
    supported_fields = []
    if api_obj.get_list():
        for field in fields:
            try:
                api_response = getattr(api_obj, "get_" + field)()
            except (MethodNotFound, WebFault):
                pass
            else:
                lists.append(api_response)
                supported_fields.append(field)
        for i, j in enumerate(api_obj.get_list()):
            temp = {}
            temp.update([(item[0], item[1][i]) for item in zip(supported_fields, lists)])
            result_dict[j] = temp
    return result_dict


def generate_simple_dict(api_obj, fields):
    result_dict = {}
    for field in fields:
        try:
            api_response = getattr(api_obj, "get_" + field)()
        except (MethodNotFound, WebFault):
            pass
        else:
            result_dict[field] = api_response
    return result_dict


def generate_interface_dict(f5, regex):
    interfaces = Interfaces(f5.get_api(), regex)
    fields = ['active_media', 'actual_flow_control', 'bundle_state',
              'description', 'dual_media_state', 'enabled_state', 'if_index',
              'learning_mode', 'lldp_admin_status', 'lldp_tlvmap',
              'mac_address', 'media', 'media_option', 'media_option_sfp',
              'media_sfp', 'media_speed', 'media_status', 'mtu',
              'phy_master_slave_mode', 'prefer_sfp_state', 'flow_control',
              'sflow_poll_interval', 'sflow_poll_interval_global',
              'sfp_media_state', 'stp_active_edge_port_state',
              'stp_enabled_state', 'stp_link_type',
              'stp_protocol_detection_reset_state']
    return generate_dict(interfaces, fields)


def generate_self_ip_dict(f5, regex):
    self_ips = SelfIPs(f5.get_api(), regex)
    fields = ['address', 'allow_access_list', 'description',
              'enforced_firewall_policy', 'floating_state', 'fw_rule',
              'netmask', 'staged_firewall_policy', 'traffic_group',
              'vlan', 'is_traffic_group_inherited']
    return generate_dict(self_ips, fields)


def generate_trunk_dict(f5, regex):
    trunks = Trunks(f5.get_api(), regex)
    fields = ['active_lacp_state', 'configured_member_count', 'description',
              'distribution_hash_option', 'interface', 'lacp_enabled_state',
              'lacp_timeout_option', 'link_selection_policy', 'media_speed',
              'media_status', 'operational_member_count', 'stp_enabled_state',
              'stp_protocol_detection_reset_state']
    return generate_dict(trunks, fields)


def generate_vlan_dict(f5, regex):
    vlans = Vlans(f5.get_api(), regex)
    fields = ['auto_lasthop', 'cmp_hash_algorithm', 'description',
              'dynamic_forwarding', 'failsafe_action', 'failsafe_state',
              'failsafe_timeout', 'if_index', 'learning_mode',
              'mac_masquerade_address', 'member', 'mtu',
              'sflow_poll_interval', 'sflow_poll_interval_global',
              'sflow_sampling_rate', 'sflow_sampling_rate_global',
              'source_check_state', 'true_mac_address', 'vlan_id']
    return generate_dict(vlans, fields)


def generate_vs_dict(f5, regex):
    virtual_servers = VirtualServers(f5.get_api(), regex)
    fields = ['actual_hardware_acceleration', 'authentication_profile',
              'auto_lasthop', 'bw_controller_policy', 'clone_pool',
              'cmp_enable_mode', 'connection_limit', 'connection_mirror_state',
              'default_pool_name', 'description', 'destination',
              'enabled_state', 'enforced_firewall_policy',
              'fallback_persistence_profile', 'fw_rule', 'gtm_score',
              'last_hop_pool', 'nat64_state', 'object_status',
              'persistence_profile', 'profile', 'protocol',
              'rate_class', 'rate_limit', 'rate_limit_destination_mask',
              'rate_limit_mode', 'rate_limit_source_mask', 'related_rule',
              'rule', 'security_log_profile', 'snat_pool', 'snat_type',
              'source_address', 'source_address_translation_lsn_pool',
              'source_address_translation_snat_pool',
              'source_address_translation_type', 'source_port_behavior',
              'staged_firewall_policy', 'translate_address_state',
              'translate_port_state', 'type', 'vlan', 'wildmask',
              'name']
    return generate_dict(virtual_servers, fields)


def generate_pool_dict(f5, regex):
    pools = Pools(f5.get_api(), regex)
    fields = ['action_on_service_down', 'active_member_count',
              'aggregate_dynamic_ratio', 'allow_nat_state',
              'allow_snat_state', 'client_ip_tos', 'client_link_qos',
              'description', 'gateway_failsafe_device',
              'ignore_persisted_weight_state', 'lb_method', 'member',
              'minimum_active_member', 'minimum_up_member',
              'minimum_up_member_action', 'minimum_up_member_enabled_state',
              'monitor_association', 'monitor_instance', 'object_status',
              'profile', 'queue_depth_limit',
              'queue_on_connection_limit_state', 'queue_time_limit',
              'reselect_tries', 'server_ip_tos', 'server_link_qos',
              'simple_timeout', 'slow_ramp_time', 'name']
    return generate_dict(pools, fields)


def generate_device_dict(f5, regex):
    devices = Devices(f5.get_api(), regex)
    fields = ['active_modules', 'base_mac_address', 'blade_addresses',
              'build', 'chassis_id', 'chassis_type', 'comment',
              'configsync_address', 'contact', 'description', 'edition',
              'failover_state', 'hostname', 'inactive_modules', 'location',
              'management_address', 'marketing_name', 'multicast_address',
              'optional_modules', 'platform_id', 'primary_mirror_address',
              'product', 'secondary_mirror_address', 'software_version',
              'timelimited_modules', 'timezone', 'unicast_addresses']
    return generate_dict(devices, fields)


def generate_device_group_dict(f5, regex):
    device_groups = DeviceGroups(f5.get_api(), regex)
    fields = ['all_preferred_active', 'autosync_enabled_state', 'description',
              'device', 'full_load_on_sync_state',
              'incremental_config_sync_size_maximum',
              'network_failover_enabled_state', 'sync_status', 'type']
    return generate_dict(device_groups, fields)


def generate_traffic_group_dict(f5, regex):
    traffic_groups = TrafficGroups(f5.get_api(), regex)
    fields = ['auto_failback_enabled_state', 'auto_failback_time',
              'default_device', 'description', 'ha_load_factor',
              'ha_order', 'is_floating', 'mac_masquerade_address',
              'unit_id']
    return generate_dict(traffic_groups, fields)


def generate_rule_dict(f5, regex):
    rules = Rules(f5.get_api(), regex)
    fields = ['definition', 'description', 'ignore_vertification',
              'verification_status']
    return generate_dict(rules, fields)


def generate_node_dict(f5, regex):
    nodes = Nodes(f5.get_api(), regex)
    fields = ['name', 'address', 'connection_limit', 'description', 'dynamic_ratio',
              'monitor_instance', 'monitor_rule', 'monitor_status',
              'object_status', 'rate_limit', 'ratio', 'session_status']
    return generate_dict(nodes, fields)


def generate_virtual_address_dict(f5, regex):
    virtual_addresses = VirtualAddresses(f5.get_api(), regex)
    fields = ['address', 'arp_state', 'auto_delete_state', 'connection_limit',
              'description', 'enabled_state', 'icmp_echo_state',
              'is_floating_state', 'netmask', 'object_status',
              'route_advertisement_state', 'traffic_group']
    return generate_dict(virtual_addresses, fields)


def generate_address_class_dict(f5, regex):
    address_classes = AddressClasses(f5.get_api(), regex)
    fields = ['address_class', 'description']
    return generate_dict(address_classes, fields)


def generate_certificate_dict(f5, regex):
    certificates = Certificates(f5.get_api(), regex)
    return dict(zip(certificates.get_list(), certificates.get_certificate_list()))


def generate_key_dict(f5, regex):
    keys = Keys(f5.get_api(), regex)
    return dict(zip(keys.get_list(), keys.get_key_list()))


def generate_client_ssl_profile_dict(f5, regex):
    profiles = ProfileClientSSL(f5.get_api(), regex)
    fields = ['alert_timeout', 'allow_nonssl_state', 'authenticate_depth',
              'authenticate_once_state', 'ca_file', 'cache_size',
              'cache_timeout', 'certificate_file', 'chain_file',
              'cipher_list', 'client_certificate_ca_file', 'crl_file',
              'default_profile', 'description',
              'forward_proxy_ca_certificate_file', 'forward_proxy_ca_key_file',
              'forward_proxy_ca_passphrase',
              'forward_proxy_certificate_extension_include',
              'forward_proxy_certificate_lifespan',
              'forward_proxy_enabled_state',
              'forward_proxy_lookup_by_ipaddr_port_state', 'handshake_timeout',
              'key_file', 'modssl_emulation_state', 'passphrase',
              'peer_certification_mode', 'profile_mode',
              'renegotiation_maximum_record_delay', 'renegotiation_period',
              'renegotiation_state', 'renegotiation_throughput',
              'retain_certificate_state', 'secure_renegotiation_mode',
              'server_name', 'session_ticket_state', 'sni_default_state',
              'sni_require_state', 'ssl_option', 'strict_resume_state',
              'unclean_shutdown_state', 'is_base_profile', 'is_system_profile']
    return generate_dict(profiles, fields)


def generate_system_info_dict(f5):
    system_info = SystemInfo(f5.get_api())
    fields = ['base_mac_address',
              'blade_temperature', 'chassis_slot_information',
              'globally_unique_identifier', 'group_id',
              'hardware_information',
              'marketing_name',
              'product_information', 'pva_version', 'system_id',
              'system_information', 'time',
              'time_zone', 'uptime']
    return generate_simple_dict(system_info, fields)


def generate_software_list(f5):
    software = Software(f5.get_api())
    software_list = software.get_all_software_status()
    return software_list


def generate_provision_dict(f5):
    provisioned = ProvisionInfo(f5.get_api())
    fields = ['list', 'provisioned_list']
    return generate_simple_dict(provisioned, fields)


def main():
    argument_spec = f5_argument_spec
    meta_args = dict(
        session=dict(type='bool', default='no'),
        include=dict(
            type='raw',
            required=True,
            choices=[
                'address_class', 'certificate', 'client_ssl_profile', 'device',
                'device_group', 'interface', 'key', 'node', 'pool', 'provision',
                'rule', 'self_ip', 'software', 'system_info', 'traffic_group',
                'trunk', 'virtual_address', 'virtual_server', 'vlan'
            ]
        ),
        filter=dict(type='str'),
    )
    argument_spec.update(meta_args)

    module = AnsibleModule(
        argument_spec=argument_spec
    )

    if not bigsuds_found:
        module.fail_json(msg="the python suds and bigsuds modules are required")

    server = module.params['server']
    server_port = module.params['server_port']
    user = module.params['user']
    password = module.params['password']
    validate_certs = module.params['validate_certs']
    session = module.params['session']
    fact_filter = module.params['filter']

    if validate_certs:
        import ssl
        if not hasattr(ssl, 'SSLContext'):
            module.fail_json(
                msg='bigsuds does not support verifying certificates with python < 2.7.9.  Either update python or set validate_certs=False on the task'
            )

    if fact_filter:
        regex = fnmatch.translate(fact_filter)
    else:
        regex = None
    if isinstance(module.params['include'], string_types):
        includes = module.params['include'].split(',')
    else:
        includes = module.params['include']
    include = [x.lower() for x in includes]
    valid_includes = ('address_class', 'certificate', 'client_ssl_profile',
                      'device', 'device_group', 'interface', 'key', 'node',
                      'pool', 'provision', 'rule', 'self_ip', 'software',
                      'system_info', 'traffic_group', 'trunk',
                      'virtual_address', 'virtual_server', 'vlan')
    include_test = (x in valid_includes for x in include)
    if not all(include_test):
        module.fail_json(msg="Value of include must be one or more of: %s, got: %s" % (",".join(valid_includes), ",".join(include)))

    try:
        facts = {}

        if len(include) > 0:
            f5 = F5(server, user, password, session, validate_certs, server_port)
            saved_active_folder = f5.get_active_folder()
            saved_recursive_query_state = f5.get_recursive_query_state()
            if saved_active_folder != "/":
                f5.set_active_folder("/")
            if saved_recursive_query_state != "STATE_ENABLED":
                f5.enable_recursive_query_state()

            if 'interface' in include:
                facts['interface'] = generate_interface_dict(f5, regex)
            if 'self_ip' in include:
                facts['self_ip'] = generate_self_ip_dict(f5, regex)
            if 'trunk' in include:
                facts['trunk'] = generate_trunk_dict(f5, regex)
            if 'vlan' in include:
                facts['vlan'] = generate_vlan_dict(f5, regex)
            if 'virtual_server' in include:
                facts['virtual_server'] = generate_vs_dict(f5, regex)
            if 'pool' in include:
                facts['pool'] = generate_pool_dict(f5, regex)
            if 'provision' in include:
                facts['provision'] = generate_provision_dict(f5)
            if 'device' in include:
                facts['device'] = generate_device_dict(f5, regex)
            if 'device_group' in include:
                facts['device_group'] = generate_device_group_dict(f5, regex)
            if 'traffic_group' in include:
                facts['traffic_group'] = generate_traffic_group_dict(f5, regex)
            if 'rule' in include:
                facts['rule'] = generate_rule_dict(f5, regex)
            if 'node' in include:
                facts['node'] = generate_node_dict(f5, regex)
            if 'virtual_address' in include:
                facts['virtual_address'] = generate_virtual_address_dict(f5, regex)
            if 'address_class' in include:
                facts['address_class'] = generate_address_class_dict(f5, regex)
            if 'software' in include:
                facts['software'] = generate_software_list(f5)
            if 'certificate' in include:
                facts['certificate'] = generate_certificate_dict(f5, regex)
            if 'key' in include:
                facts['key'] = generate_key_dict(f5, regex)
            if 'client_ssl_profile' in include:
                facts['client_ssl_profile'] = generate_client_ssl_profile_dict(f5, regex)
            if 'system_info' in include:
                facts['system_info'] = generate_system_info_dict(f5)

            # restore saved state
            if saved_active_folder and saved_active_folder != "/":
                f5.set_active_folder(saved_active_folder)
            if saved_recursive_query_state and \
               saved_recursive_query_state != "STATE_ENABLED":
                f5.set_recursive_query_state(saved_recursive_query_state)

        result = dict(
            ansible_facts=facts,
        )
        result.update(**facts)

    except Exception as e:
        module.fail_json(msg="received exception: %s\ntraceback: %s" % (e, traceback.format_exc()))

    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2017, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'network'}


DOCUMENTATION = """
---
module: junos_lldp
version_added: "2.4"
author: "Ganesh Nalawade (@ganeshrn)"
short_description: Manage LLDP configuration on Juniper JUNOS network devices
description:
  - This module provides declarative management of LLDP service
    on Juniper JUNOS network devices.
options:
  interval:
    description:
      - Frequency at which LLDP advertisements are sent (in seconds).
  transmit_delay:
    description:
      - Specify the number of seconds the device waits before sending
        advertisements to neighbors after a change is made in local system.
  hold_multiplier:
    description:
      - Specify the number of seconds that LLDP information is held before it is
        discarded. The multiplier value is used in combination with the
        C(interval) value.
  state:
    description:
      - Value of C(present) ensures given LLDP configuration
        is present on device and LLDP is enabled, for value of C(absent)
        LLDP configuration is deleted and LLDP is in disabled state.
        Value C(enabled) ensures LLDP protocol is enabled and LLDP configuration
        if any is configured on remote device, for value of C(disabled) it ensures
        LLDP protocol is disabled any LLDP configuration if any is still present.
    default: present
    choices: ['present', 'absent', 'enabled', 'disabled']
  active:
    description:
      - Specifies whether or not the configuration is active or deactivated
    default: True
    type: bool
requirements:
  - ncclient (>=v0.5.2)
notes:
  - This module requires the netconf system service be enabled on
    the remote device being managed.
  - Tested against vSRX JUNOS version 15.1X49-D15.4, vqfx-10000 JUNOS Version 15.1X53-D60.4.
  - Recommended connection is C(netconf). See L(the Junos OS Platform Options,../network/user_guide/platform_junos.html).
  - This module also works with C(local) connections for legacy playbooks.
extends_documentation_fragment: junos
"""

EXAMPLES = """
- name: Enable LLDP service
  junos_lldp:
    state: enabled

- name: Disable LLDP service
  junos_lldp:
    state: disabled

- name: Set LLDP parameters
  junos_lldp:
    interval: 10
    hold_multiplier: 5
    transmit_delay: 30
    state: present

- name: Delete LLDP parameters
  junos_lldp:
    interval: 10
    hold_multiplier: 5
    transmit_delay: 30
    state: absent
"""

RETURN = """
diff.prepared:
  description: Configuration difference before and after applying change.
  returned: when configuration is changed and diff option is enabled.
  type: string
  sample: >
        [edit]
        +  protocols {
        +      lldp {
        +          disable;
        +      }
        +  }
"""
import collections

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.network.junos.junos import junos_argument_spec, tostring
from ansible.module_utils.network.junos.junos import load_config, map_params_to_obj, map_obj_to_ele
from ansible.module_utils.network.junos.junos import commit_configuration, discard_changes, locked_config

USE_PERSISTENT_CONNECTION = True


def validate_interval(value, module):
    if not 5 <= value <= 32768:
        module.fail_json(msg='interval must be between 5 and 32768')


def validate_hold_multiplier(value, module):
    if not 5 <= value <= 32768:
        module.fail_json(msg='hold_multiplier must be between 2 and 10')


def validate_transmit_delay(value, module):
    if not 1 <= value <= 8192:
        module.fail_json(msg='transmit_delay must be between 2 and 10')


def validate_param_values(module, obj):
    for key in obj:
        # validate the param value (if validator func exists)
        validator = globals().get('validate_%s' % key)
        if callable(validator):
            validator(module.params.get(key), module)


def main():
    """ main entry point for module execution
    """
    argument_spec = dict(
        interval=dict(type='int'),
        transmit_delay=dict(type='int'),
        hold_multiplier=dict(type='int'),
        state=dict(default='present', choices=['present', 'absent', 'enabled', 'disabled']),
        active=dict(default=True, type='bool')
    )

    argument_spec.update(junos_argument_spec)

    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True)

    warnings = list()
    result = {'changed': False}

    if warnings:
        result['warnings'] = warnings

    top = 'protocols/lldp'

    param_to_xpath_map = collections.OrderedDict()
    param_to_xpath_map.update([
        ('interval', {'xpath': 'advertisement-interval', 'leaf_only': True}),
        ('transmit_delay', {'xpath': 'transmit-delay', 'leaf_only': True}),
        ('hold_multiplier', {'xpath': 'hold-multiplier', 'leaf_only': True}),
        ('disable', {'xpath': 'disable', 'tag_only': True, 'is_key': True})
    ])

    item = module.params.copy()
    state = item.get('state')

    item['disable'] = True if state in ('disabled', 'absent') else False

    if state in ('enabled', 'disabled'):
        item['state'] = 'present'

    want = map_params_to_obj(module, param_to_xpath_map, param=item)
    ele = map_obj_to_ele(module, want, top, param=item)

    with locked_config(module):
        diff = load_config(module, tostring(ele), warnings, action='merge')

        commit = not module.check_mode
        if diff:
            if commit:
                commit_configuration(module)
            else:
                discard_changes(module)
            result['changed'] = True

            if module._diff:
                result['diff'] = {'prepared': diff}

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import optparse
import os
import sys
import yaml

from jinja2 import Environment, FileSystemLoader

DEFAULT_TEMPLATE_FILE = 'config.rst.j2'


def generate_parser():
    p = optparse.OptionParser(
        version='%prog 1.0',
        usage='usage: %prog [options]',
        description='Generate module documentation from metadata',
    )
    p.add_option("-t", "--template-file", action="store", dest="template_file", default=DEFAULT_TEMPLATE_FILE, help="directory containing Jinja2 templates")
    p.add_option("-o", "--output-dir", action="store", dest="output_dir", default='/tmp/', help="Output directory for rst files")
    p.add_option("-d", "--docs-source", action="store", dest="docs", default=None, help="Source for attribute docs")

    (options, args) = p.parse_args()

    return p


def fix_description(config_options):
    '''some descriptions are strings, some are lists. workaround it...'''

    for config_key in config_options:
        description = config_options[config_key].get('description', [])
        if isinstance(description, list):
            desc_list = description
        else:
            desc_list = [description]
        config_options[config_key]['description'] = desc_list
    return config_options


def main(args):

    parser = generate_parser()
    (options, args) = parser.parse_args()

    output_dir = os.path.abspath(options.output_dir)
    template_file_full_path = os.path.abspath(options.template_file)
    template_file = os.path.basename(template_file_full_path)
    template_dir = os.path.dirname(os.path.abspath(template_file_full_path))

    if options.docs:
        with open(options.docs) as f:
            docs = yaml.safe_load(f)
    else:
        docs = {}

    config_options = docs
    config_options = fix_description(config_options)

    env = Environment(loader=FileSystemLoader(template_dir), trim_blocks=True,)
    template = env.get_template(template_file)
    output_name = os.path.join(output_dir, template_file.replace('.j2', ''))
    temp_vars = {'config_options': config_options}

    with open(output_name, 'wb') as f:
        f.write(template.render(temp_vars).encode('utf-8'))

    return 0


if __name__ == '__main__':
    sys.exit(main(sys.argv[:]))



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2015, Matt Makai <matthew.makai@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
version_added: "2.0"
module: sendgrid
short_description: Sends an email with the SendGrid API
description:
   - "Sends an email with a SendGrid account through their API, not through
     the SMTP service."
notes:
   - "This module is non-idempotent because it sends an email through the
     external API. It is idempotent only in the case that the module fails."
   - "Like the other notification modules, this one requires an external
     dependency to work. In this case, you'll need an active SendGrid
     account."
   - "In order to use api_key, cc, bcc, attachments, from_name, html_body, headers
     you must pip install sendgrid"
   - "since 2.2 username and password are not required if you supply an api_key"
requirements:
  - sendgrid python library
options:
  username:
    description:
      - username for logging into the SendGrid account.
      - Since 2.2 it is only required if api_key is not supplied.
  password:
    description:
      - password that corresponds to the username
      - Since 2.2 it is only required if api_key is not supplied.
  from_address:
    description:
      - the address in the "from" field for the email
    required: true
  to_addresses:
    description:
      - a list with one or more recipient email addresses
    required: true
  subject:
    description:
      - the desired subject for the email
    required: true
  api_key:
    description:
      - sendgrid API key to use instead of username/password
    version_added: 2.2
  cc:
    description:
      - a list of email addresses to cc
    version_added: 2.2
  bcc:
    description:
      - a list of email addresses to bcc
    version_added: 2.2
  attachments:
    description:
      - a list of relative or explicit paths of files you want to attach (7MB limit as per SendGrid docs)
    version_added: 2.2
  from_name:
    description:
      - the name you want to appear in the from field, i.e 'John Doe'
    version_added: 2.2
  html_body:
    description:
      - whether the body is html content that should be rendered
    version_added: 2.2
    type: bool
    default: 'no'
  headers:
    description:
      - a dict to pass on as headers
    version_added: 2.2
author: "Matt Makai (@makaimc)"
'''

EXAMPLES = '''
# send an email to a single recipient that the deployment was successful
- sendgrid:
    username: "{{ sendgrid_username }}"
    password: "{{ sendgrid_password }}"
    from_address: "ansible@mycompany.com"
    to_addresses:
      - "ops@mycompany.com"
    subject: "Deployment success."
    body: "The most recent Ansible deployment was successful."
  delegate_to: localhost

# send an email to more than one recipient that the build failed
- sendgrid:
      username: "{{ sendgrid_username }}"
      password: "{{ sendgrid_password }}"
      from_address: "build@mycompany.com"
      to_addresses:
        - "ops@mycompany.com"
        - "devteam@mycompany.com"
      subject: "Build failure!."
      body: "Unable to pull source repository from Git server."
  delegate_to: localhost
'''

# =======================================
# sendgrid module support methods
#
import os

try:
    import sendgrid
    HAS_SENDGRID = True
except ImportError:
    HAS_SENDGRID = False

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.six.moves.urllib.parse import urlencode
from ansible.module_utils._text import to_bytes
from ansible.module_utils.urls import fetch_url


def post_sendgrid_api(module, username, password, from_address, to_addresses,
                      subject, body, api_key=None, cc=None, bcc=None, attachments=None,
                      html_body=False, from_name=None, headers=None):

    if not HAS_SENDGRID:
        SENDGRID_URI = "https://api.sendgrid.com/api/mail.send.json"
        AGENT = "Ansible"
        data = {'api_user': username, 'api_key': password,
                'from': from_address, 'subject': subject, 'text': body}
        encoded_data = urlencode(data)
        to_addresses_api = ''
        for recipient in to_addresses:
            recipient = to_bytes(recipient, errors='surrogate_or_strict')
            to_addresses_api += '&to[]=%s' % recipient
        encoded_data += to_addresses_api

        headers = {'User-Agent': AGENT,
                   'Content-type': 'application/x-www-form-urlencoded',
                   'Accept': 'application/json'}
        return fetch_url(module, SENDGRID_URI, data=encoded_data, headers=headers, method='POST')
    else:

        if api_key:
            sg = sendgrid.SendGridClient(api_key)
        else:
            sg = sendgrid.SendGridClient(username, password)

        message = sendgrid.Mail()
        message.set_subject(subject)

        for recip in to_addresses:
            message.add_to(recip)

        if cc:
            for recip in cc:
                message.add_cc(recip)
        if bcc:
            for recip in bcc:
                message.add_bcc(recip)

        if headers:
            message.set_headers(headers)

        if attachments:
            for f in attachments:
                name = os.path.basename(f)
                message.add_attachment(name, f)

        if from_name:
            message.set_from('%s <%s.' % (from_name, from_address))
        else:
            message.set_from(from_address)

        if html_body:
            message.set_html(body)
        else:
            message.set_text(body)

        return sg.send(message)
# =======================================
# Main
#


def main():
    module = AnsibleModule(
        argument_spec=dict(
            username=dict(required=False),
            password=dict(required=False, no_log=True),
            api_key=dict(required=False, no_log=True),
            bcc=dict(required=False, type='list'),
            cc=dict(required=False, type='list'),
            headers=dict(required=False, type='dict'),
            from_address=dict(required=True),
            from_name=dict(required=False),
            to_addresses=dict(required=True, type='list'),
            subject=dict(required=True),
            body=dict(required=True),
            html_body=dict(required=False, default=False, type='bool'),
            attachments=dict(required=False, type='list')
        ),
        supports_check_mode=True,
        mutually_exclusive=[
            ['api_key', 'password'],
            ['api_key', 'username']
        ],
        required_together=[['username', 'password']],
    )

    username = module.params['username']
    password = module.params['password']
    api_key = module.params['api_key']
    bcc = module.params['bcc']
    cc = module.params['cc']
    headers = module.params['headers']
    from_name = module.params['from_name']
    from_address = module.params['from_address']
    to_addresses = module.params['to_addresses']
    subject = module.params['subject']
    body = module.params['body']
    html_body = module.params['html_body']
    attachments = module.params['attachments']

    sendgrid_lib_args = [api_key, bcc, cc, headers, from_name, html_body, attachments]

    if any(lib_arg is not None for lib_arg in sendgrid_lib_args) and not HAS_SENDGRID:
        module.fail_json(msg='You must install the sendgrid python library if you want to use any of the following arguments: '
                             'api_key, bcc, cc, headers, from_name, html_body, attachments')

    response, info = post_sendgrid_api(module, username, password,
                                       from_address, to_addresses, subject, body, attachments=attachments,
                                       bcc=bcc, cc=cc, headers=headers, html_body=html_body, api_key=api_key)

    if not HAS_SENDGRID:
        if info['status'] != 200:
            module.fail_json(msg="unable to send email through SendGrid API: %s" % info['msg'])
    else:
        if response != 200:
            module.fail_json(msg="unable to send email through SendGrid API: %s" % info['message'])

    module.exit_json(msg=subject, changed=False)


if __name__ == '__main__':
    main()



#!/usr/bin/python

# (c) 2018, NetApp, Inc
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

DOCUMENTATION = '''
---

module: na_ontap_ucadapter
short_description: ONTAP UC adapter configuration
extends_documentation_fragment:
    - netapp.na_ontap
version_added: '2.6'
author: NetApp Ansible Team (ng-ansibleteam@netapp.com)

description:
    - modify the UC adapter mode and type taking pending type and mode into account.

options:
  state:
    description:
    - Whether the specified adapter should exist.
    required: false
    choices: ['present']
    default: 'present'

  adapter_name:
    description:
    - Specifies the adapter name.
    required: true

  node_name:
    description:
    - Specifies the adapter home node.
    required: true

  mode:
    description:
    - Specifies the mode of the adapter.

  type:
    description:
    - Specifies the fc4 type of the adapter.

'''

EXAMPLES = '''
    - name: Modify adapter
      na_ontap_adapter:
        state: present
        adapter_name: data2
        node_name: laurentn-vsim1
        mode: fc
        type: target
        hostname: "{{ netapp_hostname }}"
        username: "{{ netapp_username }}"
        password: "{{ netapp_password }}"

'''

RETURN = '''
'''

import traceback

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils._text import to_native
import ansible.module_utils.netapp as netapp_utils

HAS_NETAPP_LIB = netapp_utils.has_netapp_lib()


class NetAppOntapadapter(object):
    ''' object to describe  adapter info '''

    def __init__(self):

        self.argument_spec = netapp_utils.na_ontap_host_argument_spec()
        self.argument_spec.update(dict(
            state=dict(required=False, choices=['present'], default='present'),
            adapter_name=dict(required=True, type='str'),
            node_name=dict(required=True, type='str'),
            mode=dict(required=False, type='str'),
            type=dict(required=False, type='str'),
        ))

        self.module = AnsibleModule(
            argument_spec=self.argument_spec,
            supports_check_mode=True
        )

        params = self.module.params

        # set up state variables
        self.state = params['state']
        self.adapter_name = params['adapter_name']
        self.node_name = params['node_name']
        self.mode = params['mode']
        self.type = params['type']

        if HAS_NETAPP_LIB is False:
            self.module.fail_json(msg="the python NetApp-Lib module is required")
        else:
            self.server = netapp_utils.setup_na_ontap_zapi(module=self.module)

    def get_adapter(self):
        """
        Return details about the adapter
        :param:
            name : Name of the name of the adapter

        :return: Details about the adapter. None if not found.
        :rtype: dict
        """
        adapter_info = netapp_utils.zapi.NaElement('ucm-adapter-get')
        adapter_info.add_new_child('adapter-name', self.adapter_name)
        adapter_info.add_new_child('node-name', self.node_name)
        result = self.server.invoke_successfully(adapter_info, True)
        return_value = None
        adapter_attributes = result.get_child_by_name('attributes').\
            get_child_by_name('uc-adapter-info')
        return_value = {
            'mode': adapter_attributes.get_child_content('mode'),
            'pending-mode': adapter_attributes.get_child_content('pending-mode'),
            'type': adapter_attributes.get_child_content('fc4-type'),
            'pending-type': adapter_attributes.get_child_content('pending-fc4-type'),
            'status': adapter_attributes.get_child_content('status'),
        }
        return return_value

    def modify_adapter(self):
        """
        Modify the adapter.
        """
        params = {'adapter-name': self.adapter_name,
                  'node-name': self.node_name}
        if self.type is not None:
            params['fc4-type'] = self.type
        if self.mode is not None:
            params['mode'] = self.mode
        adapter_modify = netapp_utils.zapi.NaElement.create_node_with_children(
            'ucm-adapter-modify', ** params)
        try:
            self.server.invoke_successfully(adapter_modify,
                                            enable_tunneling=True)
        except netapp_utils.zapi.NaApiError as e:
            self.module.fail_json(msg='Error modifying adapter %s: %s' % (self.adapter_name, to_native(e)),
                                  exception=traceback.format_exc())

    def offline_adapter(self):
        """
        Bring a Fibre Channel target adapter offline.
        """
        adapter_offline = netapp_utils.zapi.NaElement('fcp-adapter-config-down')
        adapter_offline.add_new_child('fcp-adapter', self.adapter_name)
        adapter_offline.add_new_child('node', self.node_name)
        try:
            self.server.invoke_successfully(adapter_offline,
                                            enable_tunneling=True)
        except netapp_utils.zapi.NaApiError as e:
            self.module.fail_json(msg='Error trying to offline fc-adapter %s: %s' % (self.adapter_name, to_native(e)),
                                  exception=traceback.format_exc())

    def online_adapter(self):
        """
        Bring a Fibre Channel target adapter online.
        """
        adapter_online = netapp_utils.zapi.NaElement('fcp-adapter-config-up')
        adapter_online.add_new_child('fcp-adapter', self.adapter_name)
        adapter_online.add_new_child('node', self.node_name)
        try:
            self.server.invoke_successfully(adapter_online,
                                            enable_tunneling=True)
        except netapp_utils.zapi.NaApiError as e:
            self.module.fail_json(msg='Error trying to online fc-adapter %s: %s' % (self.adapter_name, to_native(e)),
                                  exception=traceback.format_exc())

    def apply(self):
        ''' calling all adapter features '''
        changed = False
        results = netapp_utils.get_cserver(self.server)
        cserver = netapp_utils.setup_na_ontap_zapi(module=self.module, vserver=results)
        netapp_utils.ems_log_event("na_ontap_ucadapter", cserver)
        adapter_detail = self.get_adapter()

        def need_to_change(expected, pending, current):
            if expected is None:
                return False
            if pending is not None:
                return pending != expected
            if current is not None:
                return current != expected
            return False

        if adapter_detail:
            changed = need_to_change(self.type, adapter_detail['pending-type'], adapter_detail['type']) or \
                need_to_change(self.mode, adapter_detail['pending-mode'], adapter_detail['mode'])

        if changed:
            if self.module.check_mode:
                pass
            else:
                self.offline_adapter()
                self.modify_adapter()
                self.online_adapter()

        self.module.exit_json(changed=changed)


def main():
    adapter = NetAppOntapadapter()
    adapter.apply()


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright: (c) 2018, Rob White (@wimnat)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = '''
---
module: elb_network_lb
short_description: Manage a Network Load Balancer
description:
    - Manage an AWS Network Elastic Load Balancer. See
      U(https://aws.amazon.com/blogs/aws/new-network-load-balancer-effortless-scaling-to-millions-of-requests-per-second/) for details.
version_added: "2.6"
requirements: [ boto3 ]
author: "Rob White (@wimnat)"
options:
  cross_zone_load_balancing:
    description:
      - Indicates whether cross-zone load balancing is enabled.
    required: false
    default: no
    type: bool
  deletion_protection:
    description:
      - Indicates whether deletion protection for the ELB is enabled.
    required: false
    default: no
    type: bool
  listeners:
    description:
      - A list of dicts containing listeners to attach to the ELB. See examples for detail of the dict required. Note that listener keys
        are CamelCased.
    required: false
  name:
    description:
      - The name of the load balancer. This name must be unique within your AWS account, can have a maximum of 32 characters, must contain only alphanumeric
        characters or hyphens, and must not begin or end with a hyphen.
    required: true
  purge_listeners:
    description:
      - If yes, existing listeners will be purged from the ELB to match exactly what is defined by I(listeners) parameter. If the I(listeners) parameter is
        not set then listeners will not be modified
    default: yes
    type: bool
  purge_tags:
    description:
      - If yes, existing tags will be purged from the resource to match exactly what is defined by I(tags) parameter. If the I(tags) parameter is not set then
        tags will not be modified.
    required: false
    default: yes
    type: bool
  subnet_mappings:
    description:
      - A list of dicts containing the IDs of the subnets to attach to the load balancer. You can also specify the allocation ID of an Elastic IP
        to attach to the load balancer. You can specify one Elastic IP address per subnet. This parameter is mutually exclusive with I(subnets)
    required: false
  subnets:
    description:
      - A list of the IDs of the subnets to attach to the load balancer. You can specify only one subnet per Availability Zone. You must specify subnets from
        at least two Availability Zones. Required if state=present. This parameter is mutually exclusive with I(subnet_mappings)
    required: false
  scheme:
    description:
      - Internet-facing or internal load balancer. An ELB scheme can not be modified after creation.
    required: false
    default: internet-facing
    choices: [ 'internet-facing', 'internal' ]
  state:
    description:
      - Create or destroy the load balancer.
    required: true
    choices: [ 'present', 'absent' ]
  tags:
    description:
      - A dictionary of one or more tags to assign to the load balancer.
    required: false
  wait:
    description:
      - Whether or not to wait for the network load balancer to reach the desired state.
    type: bool
  wait_timeout:
    description:
      - The duration in seconds to wait, used in conjunction with I(wait).
extends_documentation_fragment:
    - aws
    - ec2
notes:
  - Listeners are matched based on port. If a listener's port is changed then a new listener will be created.
  - Listener rules are matched based on priority. If a rule's priority is changed then a new rule will be created.
'''

EXAMPLES = '''
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Create an ELB and attach a listener
- elb_network_lb:
    name: myelb
    subnets:
      - subnet-012345678
      - subnet-abcdef000
    listeners:
      - Protocol: TCP # Required. The protocol for connections from clients to the load balancer (Only TCP is available) (case-sensitive).
        Port: 80 # Required. The port on which the load balancer is listening.
        DefaultActions:
          - Type: forward # Required. Only 'forward' is accepted at this time
            TargetGroupName: mytargetgroup # Required. The name of the target group
    state: present

# Create an ELB with an attached Elastic IP address
- elb_network_lb:
    name: myelb
    subnet_mappings:
      - SubnetId: subnet-012345678
        AllocationId: eipalloc-aabbccdd
    listeners:
      - Protocol: TCP # Required. The protocol for connections from clients to the load balancer (Only TCP is available) (case-sensitive).
        Port: 80 # Required. The port on which the load balancer is listening.
        DefaultActions:
          - Type: forward # Required. Only 'forward' is accepted at this time
            TargetGroupName: mytargetgroup # Required. The name of the target group
    state: present

# Remove an ELB
- elb_network_lb:
    name: myelb
    state: absent

'''

RETURN = '''
availability_zones:
    description: The Availability Zones for the load balancer.
    returned: when state is present
    type: list
    sample: "[{'subnet_id': 'subnet-aabbccddff', 'zone_name': 'ap-southeast-2a', 'load_balancer_addresses': []}]"
canonical_hosted_zone_id:
    description: The ID of the Amazon Route 53 hosted zone associated with the load balancer.
    returned: when state is present
    type: string
    sample: ABCDEF12345678
created_time:
    description: The date and time the load balancer was created.
    returned: when state is present
    type: string
    sample: "2015-02-12T02:14:02+00:00"
deletion_protection_enabled:
    description: Indicates whether deletion protection is enabled.
    returned: when state is present
    type: string
    sample: true
dns_name:
    description: The public DNS name of the load balancer.
    returned: when state is present
    type: string
    sample: internal-my-elb-123456789.ap-southeast-2.elb.amazonaws.com
idle_timeout_timeout_seconds:
    description: The idle timeout value, in seconds.
    returned: when state is present
    type: string
    sample: 60
ip_address_type:
    description:  The type of IP addresses used by the subnets for the load balancer.
    returned: when state is present
    type: string
    sample: ipv4
listeners:
    description: Information about the listeners.
    returned: when state is present
    type: complex
    contains:
        listener_arn:
            description: The Amazon Resource Name (ARN) of the listener.
            returned: when state is present
            type: string
            sample: ""
        load_balancer_arn:
            description: The Amazon Resource Name (ARN) of the load balancer.
            returned: when state is present
            type: string
            sample: ""
        port:
            description: The port on which the load balancer is listening.
            returned: when state is present
            type: int
            sample: 80
        protocol:
            description: The protocol for connections from clients to the load balancer.
            returned: when state is present
            type: string
            sample: HTTPS
        certificates:
            description: The SSL server certificate.
            returned: when state is present
            type: complex
            contains:
                certificate_arn:
                    description: The Amazon Resource Name (ARN) of the certificate.
                    returned: when state is present
                    type: string
                    sample: ""
        ssl_policy:
            description: The security policy that defines which ciphers and protocols are supported.
            returned: when state is present
            type: string
            sample: ""
        default_actions:
            description: The default actions for the listener.
            returned: when state is present
            type: string
            contains:
                type:
                    description: The type of action.
                    returned: when state is present
                    type: string
                    sample: ""
                target_group_arn:
                    description: The Amazon Resource Name (ARN) of the target group.
                    returned: when state is present
                    type: string
                    sample: ""
load_balancer_arn:
    description: The Amazon Resource Name (ARN) of the load balancer.
    returned: when state is present
    type: string
    sample: arn:aws:elasticloadbalancing:ap-southeast-2:0123456789:loadbalancer/app/my-elb/001122334455
load_balancer_name:
    description: The name of the load balancer.
    returned: when state is present
    type: string
    sample: my-elb
load_balancing_cross_zone_enabled:
    description: Indicates whether cross-zone load balancing is enabled.
    returned: when state is present
    type: string
    sample: true
scheme:
    description: Internet-facing or internal load balancer.
    returned: when state is present
    type: string
    sample: internal
state:
    description: The state of the load balancer.
    returned: when state is present
    type: dict
    sample: "{'code': 'active'}"
tags:
    description: The tags attached to the load balancer.
    returned: when state is present
    type: dict
    sample: "{
        'Tag': 'Example'
    }"
type:
    description: The type of load balancer.
    returned: when state is present
    type: string
    sample: network
vpc_id:
    description: The ID of the VPC for the load balancer.
    returned: when state is present
    type: string
    sample: vpc-0011223344
'''

from ansible.module_utils.aws.core import AnsibleAWSModule
from ansible.module_utils.ec2 import camel_dict_to_snake_dict, boto3_tag_list_to_ansible_dict, compare_aws_tags
from ansible.module_utils.aws.elbv2 import NetworkLoadBalancer, ELBListeners, ELBListener


def create_or_update_elb(elb_obj):
    """Create ELB or modify main attributes. json_exit here"""

    if elb_obj.elb:
        # ELB exists so check subnets, security groups and tags match what has been passed

        # Subnets
        if not elb_obj.compare_subnets():
            elb_obj.modify_subnets()

        # Tags - only need to play with tags if tags parameter has been set to something
        if elb_obj.tags is not None:

            # Delete necessary tags
            tags_need_modify, tags_to_delete = compare_aws_tags(boto3_tag_list_to_ansible_dict(elb_obj.elb['tags']),
                                                                boto3_tag_list_to_ansible_dict(elb_obj.tags), elb_obj.purge_tags)
            if tags_to_delete:
                elb_obj.delete_tags(tags_to_delete)

            # Add/update tags
            if tags_need_modify:
                elb_obj.modify_tags()

    else:
        # Create load balancer
        elb_obj.create_elb()

    # ELB attributes
    elb_obj.update_elb_attributes()
    elb_obj.modify_elb_attributes()

    # Listeners
    listeners_obj = ELBListeners(elb_obj.connection, elb_obj.module, elb_obj.elb['LoadBalancerArn'])

    listeners_to_add, listeners_to_modify, listeners_to_delete = listeners_obj.compare_listeners()

    # Delete listeners
    for listener_to_delete in listeners_to_delete:
        listener_obj = ELBListener(elb_obj.connection, elb_obj.module, listener_to_delete, elb_obj.elb['LoadBalancerArn'])
        listener_obj.delete()
        listeners_obj.changed = True

    # Add listeners
    for listener_to_add in listeners_to_add:
        listener_obj = ELBListener(elb_obj.connection, elb_obj.module, listener_to_add, elb_obj.elb['LoadBalancerArn'])
        listener_obj.add()
        listeners_obj.changed = True

    # Modify listeners
    for listener_to_modify in listeners_to_modify:
        listener_obj = ELBListener(elb_obj.connection, elb_obj.module, listener_to_modify, elb_obj.elb['LoadBalancerArn'])
        listener_obj.modify()
        listeners_obj.changed = True

    # If listeners changed, mark ELB as changed
    if listeners_obj.changed:
        elb_obj.changed = True

    # Get the ELB again
    elb_obj.update()

    # Get the ELB listeners again
    listeners_obj.update()

    # Update the ELB attributes
    elb_obj.update_elb_attributes()

    # Convert to snake_case and merge in everything we want to return to the user
    snaked_elb = camel_dict_to_snake_dict(elb_obj.elb)
    snaked_elb.update(camel_dict_to_snake_dict(elb_obj.elb_attributes))
    snaked_elb['listeners'] = []
    for listener in listeners_obj.current_listeners:
        snaked_elb['listeners'].append(camel_dict_to_snake_dict(listener))

    # Change tags to ansible friendly dict
    snaked_elb['tags'] = boto3_tag_list_to_ansible_dict(snaked_elb['tags'])

    elb_obj.module.exit_json(changed=elb_obj.changed, **snaked_elb)


def delete_elb(elb_obj):

    if elb_obj.elb:
        elb_obj.delete()

    elb_obj.module.exit_json(changed=elb_obj.changed)


def main():

    argument_spec = (
        dict(
            cross_zone_load_balancing=dict(type='bool'),
            deletion_protection=dict(type='bool'),
            listeners=dict(type='list',
                           elements='dict',
                           options=dict(
                               Protocol=dict(type='str', required=True),
                               Port=dict(type='int', required=True),
                               SslPolicy=dict(type='str'),
                               Certificates=dict(type='list'),
                               DefaultActions=dict(type='list', required=True)
                           )
                           ),
            name=dict(required=True, type='str'),
            purge_listeners=dict(default=True, type='bool'),
            purge_tags=dict(default=True, type='bool'),
            subnets=dict(type='list'),
            subnet_mappings=dict(type='list'),
            scheme=dict(default='internet-facing', choices=['internet-facing', 'internal']),
            state=dict(choices=['present', 'absent'], type='str'),
            tags=dict(type='dict'),
            wait_timeout=dict(type='int'),
            wait=dict(type='bool')
        )
    )

    module = AnsibleAWSModule(argument_spec=argument_spec,
                              mutually_exclusive=[['subnets', 'subnet_mappings']])

    # Check for subnets or subnet_mappings if state is present
    state = module.params.get("state")
    if state == 'present':
        if module.params.get("subnets") is None and module.params.get("subnet_mappings") is None:
            module.fail_json(msg="'subnets' or 'subnet_mappings' is required when state=present")

    # Quick check of listeners parameters
    listeners = module.params.get("listeners")
    if listeners is not None:
        for listener in listeners:
            for key in listener.keys():
                if key == 'Protocol' and listener[key] != 'TCP':
                    module.fail_json(msg="'Protocol' must be 'TCP'")

    connection = module.client('elbv2')
    connection_ec2 = module.client('ec2')

    elb = NetworkLoadBalancer(connection, connection_ec2, module)

    if state == 'present':
        create_or_update_elb(elb)
    else:
        delete_elb(elb)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2017, Thierry Sallé (@seuf)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

ANSIBLE_METADATA = {
    'status': ['preview'],
    'supported_by': 'community',
    'metadata_version': '1.1'
}

DOCUMENTATION = '''
---
module: grafana_datasource
author:
  - Thierry Sallé (@seuf)
version_added: "2.5"
short_description: Manage Grafana datasources
description:
  - Create/update/delete Grafana datasources via API.
options:
  grafana_url:
    description:
      - The Grafana URL.
    required: true
  name:
    description:
      - The name of the datasource.
    required: true
  ds_type:
    description:
     - The type of the datasource.
    required: true
    choices: [ graphite, prometheus, elasticsearch, influxdb, opentsdb, mysql, postgres, alexanderzobnin-zabbix-datasource]
  url:
    description:
      - The URL of the datasource.
    required: true
  access:
    description:
      - The access mode for this datasource.
    choices: [ direct, proxy ]
    default: proxy
  grafana_user:
    description:
      - The Grafana API user.
    default: admin
  grafana_password:
    description:
      - The Grafana API password.
    default: admin
  grafana_api_key:
    description:
      - The Grafana API key.
      - If set, C(grafana_user) and C(grafana_password) will be ignored.
  database:
    description:
      - Name of the database for the datasource.
      - This options is required when the C(ds_type) is C(influxdb), C(elasticsearch) (index name), C(mysql) or C(postgres).
    required: false
  user:
    description:
      - The datasource login user for influxdb datasources.
  password:
    description:
      - The datasource password
  basic_auth_user:
    description:
      - The datasource basic auth user.
      - Setting this option with basic_auth_password will enable basic auth.
  basic_auth_password:
    description:
      - The datasource basic auth password, when C(basic auth) is C(yes).
  with_credentials:
    description:
      - Whether credentials such as cookies or auth headers should be sent with cross-site requests.
    type: bool
    default: 'no'
  tls_client_cert:
    description:
      - The client TLS certificate.
      - If C(tls_client_cert) and C(tls_client_key) are set, this will enable TLS authentication.
      - Starts with ----- BEGIN CERTIFICATE -----
  tls_client_key:
    description:
      - The client TLS private key
      - Starts with ----- BEGIN RSA PRIVATE KEY -----
  tls_ca_cert:
    description:
      - The TLS CA certificate for self signed certificates.
      - Only used when C(tls_client_cert) and C(tls_client_key) are set.
  tls_skip_verify:
    description:
      - Skip the TLS datasource certificate verification.
    type: bool
    default: False
    version_added: 2.6
  is_default:
    description:
      - Make this datasource the default one.
    type: bool
    default: 'no'
  org_id:
    description:
      - Grafana Organisation ID in which the datasource should be created.
      - Not used when C(grafana_api_key) is set, because the C(grafana_api_key) only belong to one organisation.
    default: 1
  state:
    description:
      - Status of the datasource
    choices: [ absent, present ]
    default: present
  es_version:
    description:
      - Elasticsearch version (for C(ds_type = elasticsearch) only)
      - Version 56 is for elasticsearch 5.6+ where tou can specify the C(max_concurrent_shard_requests) option.
    choices: [ 2, 5, 56 ]
    default: 5
  max_concurrent_shard_requests:
    description:
      - Starting with elasticsearch 5.6, you can specify the max concurrent shard per requests.
    default: 256
  time_field:
    description:
      - Name of the time field in elasticsearch ds.
      - For example C(@timestamp)
    default: timestamp
  time_interval:
    description:
      - Minimum group by interval for C(influxdb) or C(elasticsearch) datasources.
      - for example C(>10s)
  interval:
    description:
      - For elasticsearch C(ds_type), this is the index pattern used.
    choices: ['', 'Hourly', 'Daily', 'Weekly', 'Monthly', 'Yearly']
  tsdb_version:
    description:
      - The opentsdb version.
      - Use C(1) for <=2.1, C(2) for ==2.2, C(3) for ==2.3.
    choices: [ 1, 2, 3 ]
    default: 1
  tsdb_resolution:
    description:
      - The opentsdb time resolution.
    choices: [ millisecond, second ]
    default: second
  sslmode:
    description:
      - SSL mode for C(postgres) datasoure type.
    choices: [ disable, require, verify-ca, verify-full ]
  trends:
    required: false
    description:
      - Use trends or not for zabbix datasource type
    type: bool
    version_added: 2.6
  validate_certs:
    description:
      - Whether to validate the Grafana certificate.
    type: bool
    default: 'yes'
'''

EXAMPLES = '''
---
- name: Create elasticsearch datasource
  grafana_datasource:
    name: "datasource-elastic"
    grafana_url: "https://grafana.company.com"
    grafana_user: "admin"
    grafana_password: "xxxxxx"
    org_id: "1"
    ds_type: "elasticisearch"
    url: "https://elastic.company.com:9200"
    database: "[logstash_]YYYY.MM.DD"
    basic_auth_user: "grafana"
    basic_auth_password: "******"
    time_field: "@timestamp"
    time_interval: "1m"
    interval: "Daily"
    es_version: 56
    max_concurrent_shard_requests: 42
    tls_ca_cert: "/etc/ssl/certs/ca.pem"

- name: Create influxdb datasource
  grafana_datasource:
    name: "datasource-influxdb"
    grafana_url: "https://grafana.company.com"
    grafana_user: "admin"
    grafana_password: "xxxxxx"
    org_id: "1"
    ds_type: "influxdb"
    url: "https://influx.company.com:8086"
    database: "telegraf"
    time_interval: ">10s"
    tls_ca_cert: "/etc/ssl/certs/ca.pem"
'''

RETURN = '''
---
name:
  description: name of the datasource created.
  returned: success
  type: string
  sample: test-ds
id:
  description: Id of the datasource
  returned: success
  type: int
  sample: 42
before:
  description: datasource returned by grafana api
  returned: changed
  type: dict
  sample: { "access": "proxy",
        "basicAuth": false,
        "database": "test_*",
        "id": 1035,
        "isDefault": false,
        "jsonData": {
            "esVersion": 5,
            "timeField": "@timestamp",
            "timeInterval": "1m",
        },
        "name": "grafana_datasource_test",
        "orgId": 1,
        "type": "elasticsearch",
        "url": "http://elastic.company.com:9200",
        "user": "",
        "password": "",
        "withCredentials": false }
after:
  description: datasource updated by module
  returned: changed
  type: dict
  sample: { "access": "proxy",
        "basicAuth": false,
        "database": "test_*",
        "id": 1035,
        "isDefault": false,
        "jsonData": {
            "esVersion": 5,
            "timeField": "@timestamp",
            "timeInterval": "10s",
        },
        "name": "grafana_datasource_test",
        "orgId": 1,
        "type": "elasticsearch",
        "url": "http://elastic.company.com:9200",
        "user": "",
        "password": "",
        "withCredentials": false }
'''

import json
import base64

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url
from ansible.module_utils._text import to_bytes

__metaclass__ = type


class GrafanaAPIException(Exception):
    pass


def grafana_switch_organisation(module, grafana_url, org_id, headers):
    r, info = fetch_url(module, '%s/api/user/using/%d' % (grafana_url, org_id), headers=headers, method='POST')
    if info['status'] != 200:
        raise GrafanaAPIException('Unable to switch to organization %s : %s' % (org_id, info))


def grafana_datasource_exists(module, grafana_url, name, headers):
    datasource_exists = False
    ds = {}
    r, info = fetch_url(module, '%s/api/datasources/name/%s' % (grafana_url, name), headers=headers, method='GET')
    if info['status'] == 200:
        datasource_exists = True
        ds = json.loads(r.read())
    elif info['status'] == 404:
        datasource_exists = False
    else:
        raise GrafanaAPIException('Unable to get datasource %s : %s' % (name, info))

    return datasource_exists, ds


def grafana_create_datasource(module, data):

    # define data payload for grafana API
    payload = {'orgId': data['org_id'],
               'name': data['name'],
               'type': data['ds_type'],
               'access': data['access'],
               'url': data['url'],
               'database': data['database'],
               'withCredentials': data['with_credentials'],
               'isDefault': data['is_default'],
               'user': data['user'],
               'password': data['password']}

    # define basic auth
    if 'basic_auth_user' in data and data['basic_auth_user'] and 'basic_auth_password' in data and data['basic_auth_password']:
        payload['basicAuth'] = True
        payload['basicAuthUser'] = data['basic_auth_user']
        payload['basicAuthPassword'] = data['basic_auth_password']
    else:
        payload['basicAuth'] = False

    # define tls auth
    json_data = {}
    if data.get('tls_client_cert') and data.get('tls_client_key'):
        json_data['tlsAuth'] = True
        if data.get('tls_ca_cert'):
            payload['secureJsonData'] = {
                'tlsCACert': data['tls_ca_cert'],
                'tlsClientCert': data['tls_client_cert'],
                'tlsClientKey': data['tls_client_key']
            }
            json_data['tlsAuthWithCACert'] = True
        else:
            payload['secureJsonData'] = {
                'tlsClientCert': data['tls_client_cert'],
                'tlsClientKey': data['tls_client_key']
            }
    else:
        json_data['tlsAuth'] = False
        json_data['tlsAuthWithCACert'] = False
        if data.get('tls_ca_cert'):
            payload['secureJsonData'] = {
                'tlsCACert': data['tls_ca_cert']
            }

    if data.get('tls_skip_verify'):
        json_data['tlsSkipVerify'] = True

    # datasource type related parameters
    if data['ds_type'] == 'elasticsearch':
        json_data['esVersion'] = data['es_version']
        json_data['timeField'] = data['time_field']
        if data.get('interval'):
            json_data['interval'] = data['interval']
        if data['es_version'] >= 56:
            json_data['maxConcurrentShardRequests'] = data['max_concurrent_shard_requests']

    if data['ds_type'] == 'elasticsearch' or data['ds_type'] == 'influxdb':
        if data.get('time_interval'):
            json_data['timeInterval'] = data['time_interval']

    if data['ds_type'] == 'opentsdb':
        json_data['tsdbVersion'] = data['tsdb_version']
        if data['tsdb_resolution'] == 'second':
            json_data['tsdbResolution'] = 1
        else:
            json_data['tsdbResolution'] = 2

    if data['ds_type'] == 'postgres':
        json_data['sslmode'] = data['sslmode']

    if data['ds_type'] == 'alexanderzobnin-zabbix-datasource':
        if data.get('trends'):
            json_data['trends'] = True

    payload['jsonData'] = json_data

    # define http header
    headers = {'content-type': 'application/json; charset=utf8'}
    if 'grafana_api_key' in data and data['grafana_api_key'] is not None:
        headers['Authorization'] = "Bearer %s" % data['grafana_api_key']
    else:
        auth = base64.b64encode(to_bytes('%s:%s' % (data['grafana_user'], data['grafana_password'])).replace('\n', ''))
        headers['Authorization'] = 'Basic %s' % auth
        grafana_switch_organisation(module, data['grafana_url'], data['org_id'], headers)

    # test if datasource already exists
    datasource_exists, ds = grafana_datasource_exists(module, data['grafana_url'], data['name'], headers=headers)

    result = {}
    if datasource_exists is True:
        del ds['typeLogoUrl']
        if ds['basicAuth'] is False:
            del ds['basicAuthUser']
            del ds['basicAuthPassword']
        if 'jsonData' in ds:
            if 'tlsAuth' in ds['jsonData'] and ds['jsonData']['tlsAuth'] is False:
                del ds['secureJsonFields']
            if 'tlsAuth' not in ds['jsonData']:
                del ds['secureJsonFields']
        payload['id'] = ds['id']
        if ds == payload:
            # unchanged
            result['name'] = data['name']
            result['id'] = ds['id']
            result['msg'] = "Datasource %s unchanged." % data['name']
            result['changed'] = False
        else:
            # update
            r, info = fetch_url(module, '%s/api/datasources/%d' % (data['grafana_url'], ds['id']), data=json.dumps(payload), headers=headers, method='PUT')
            if info['status'] == 200:
                res = json.loads(r.read())
                result['name'] = data['name']
                result['id'] = ds['id']
                result['before'] = ds
                result['after'] = payload
                result['msg'] = "Datasource %s updated %s" % (data['name'], res['message'])
                result['changed'] = True
            else:
                raise GrafanaAPIException('Unable to update the datasource id %d : %s' % (ds['id'], info))
    else:
        # create
        r, info = fetch_url(module, '%s/api/datasources' % data['grafana_url'], data=json.dumps(payload), headers=headers, method='POST')
        if info['status'] == 200:
            res = json.loads(r.read())
            result['msg'] = "Datasource %s created : %s" % (data['name'], res['message'])
            result['changed'] = True
            result['name'] = data['name']
            result['id'] = res['id']
        else:
            raise GrafanaAPIException('Unable to create the new datasource %s : %s - %s.' % (data['name'], info['status'], info))

    return result


def grafana_delete_datasource(module, data):

    # define http headers
    headers = {'content-type': 'application/json'}
    if 'grafana_api_key' in data and data['grafana_api_key']:
        headers['Authorization'] = "Bearer %s" % data['grafana_api_key']
    else:
        auth = base64.b64encode(to_bytes('%s:%s' % (data['grafana_user'], data['grafana_password'])).replace('\n', ''))
        headers['Authorization'] = 'Basic %s' % auth
        grafana_switch_organisation(module, data['grafana_url'], data['org_id'], headers)

    # test if datasource already exists
    datasource_exists, ds = grafana_datasource_exists(module, data['grafana_url'], data['name'], headers=headers)

    result = {}
    if datasource_exists is True:
        # delete
        r, info = fetch_url(module, '%s/api/datasources/name/%s' % (data['grafana_url'], data['name']), headers=headers, method='DELETE')
        if info['status'] == 200:
            res = json.loads(r.read())
            result['msg'] = "Datasource %s deleted : %s" % (data['name'], res['message'])
            result['changed'] = True
            result['name'] = data['name']
            result['id'] = 0
        else:
            raise GrafanaAPIException('Unable to update the datasource id %s : %s' % (ds['id'], info))
    else:
        # datasource does not exist, do nothing
        result = {'msg': "Datasource %s does not exist." % data['name'],
                  'changed': False,
                  'id': 0,
                  'name': data['name']}

    return result


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(required=True, type='str'),
            state=dict(choices=['present', 'absent'],
                       default='present'),
            grafana_url=dict(required=True, type='str'),
            ds_type=dict(choices=['graphite',
                                  'prometheus',
                                  'elasticsearch',
                                  'influxdb',
                                  'opentsdb',
                                  'mysql',
                                  'postgres',
                                  'alexanderzobnin-zabbix-datasource']),
            url=dict(required=True, type='str'),
            access=dict(default='proxy', choices=['proxy', 'direct']),
            grafana_user=dict(default='admin'),
            grafana_password=dict(default='admin', no_log=True),
            grafana_api_key=dict(type='str', no_log=True),
            database=dict(type='str'),
            user=dict(default='', type='str'),
            password=dict(default='', no_log=True, type='str'),
            basic_auth_user=dict(type='str'),
            basic_auth_password=dict(type='str', no_log=True),
            with_credentials=dict(default=False, type='bool'),
            tls_client_cert=dict(type='str', no_log=True),
            tls_client_key=dict(type='str', no_log=True),
            tls_ca_cert=dict(type='str', no_log=True),
            tls_skip_verify=dict(type='bool', default=False),
            is_default=dict(default=False, type='bool'),
            org_id=dict(default=1, type='int'),
            es_version=dict(type='int', default=5, choices=[2, 5, 56]),
            max_concurrent_shard_requests=dict(type='int', default=256),
            time_field=dict(default='@timestamp', type='str'),
            time_interval=dict(type='str'),
            interval=dict(type='str', choices=['', 'Hourly', 'Daily', 'Weekly', 'Monthly', 'Yearly'], default=''),
            tsdb_version=dict(type='int', default=1, choices=[1, 2, 3]),
            tsdb_resolution=dict(type='str', default='second', choices=['second', 'millisecond']),
            sslmode=dict(default='disable', choices=['disable', 'require', 'verify-ca', 'verify-full']),
            trends=dict(default=False, type='bool'),
            validate_certs=dict(type='bool', default=True)
        ),
        supports_check_mode=False,
        required_together=[['grafana_user', 'grafana_password', 'org_id'], ['tls_client_cert', 'tls_client_key']],
        mutually_exclusive=[['grafana_user', 'grafana_api_key'], ['tls_ca_cert', 'tls_skip_verify']],
        required_if=[
            ['ds_type', 'opentsdb', ['tsdb_version', 'tsdb_resolution']],
            ['ds_type', 'influxdb', ['database']],
            ['ds_type', 'elasticsearch', ['database', 'es_version', 'time_field', 'interval']],
            ['ds_type', 'mysql', ['database']],
            ['ds_type', 'postgres', ['database', 'sslmode']],
            ['es_version', 56, ['max_concurrent_shard_requests']]
        ],
    )

    try:
        if module.params['state'] == 'present':
            result = grafana_create_datasource(module, module.params)
        else:
            result = grafana_delete_datasource(module, module.params)
    except GrafanaAPIException as e:
        module.fail_json(
            failed=True,
            msg="error %s : %s " % (type(e), e)
        )
        return

    module.exit_json(
        failed=False,
        **result
    )
    return


if __name__ == '__main__':
    main()



# (c) 2016 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from nose.plugins.skip import SkipTest

try:
    from ansible.modules.network.fortimanager import fmgr_script
    from .fortimanager_module import TestFortimanagerModule
    from units.modules.utils import set_module_args
except ImportError:
    raise SkipTest("Could not load required modules for testing")

try:
    from pyFMG.fortimgr import FortiManager
except ImportError:
    raise SkipTest("FortiManager tests require pyFMG package")


class TestFmgrScriptModule(TestFortimanagerModule):

    module = fmgr_script

    def test_fmg_script_fail_connect(self):
        set_module_args(dict(host='1.1.1.1', username='admin', password='admin', adom='root', script_name='TestScript',
                             script_type='cli', script_target='remote_device', script_description='AnsibleTest',
                             script_content='get system status'))
        result = self.execute_module(failed=True)
        self.assertEqual(result['msg'], 'Connection to FortiManager Failed')

    def test_fmg_script_login_fail_host(self):
        set_module_args(dict(username='admin', password='admin', adom='root', script_name='TestScript',
                             script_type='cli', script_target='remote_device', script_description='AnsibleTest',
                             script_content='get system status'))
        result = self.execute_module(failed=True)
        self.assertEqual(result['msg'], 'missing required arguments: host')

    def test_fmg_script_login_fail_username(self):
        set_module_args(dict(host='1.1.1.1', password='admin', adom='root', script_name='TestScript',
                             script_type='cli', script_target='remote_device', script_description='AnsibleTest',
                             script_content='get system status'))
        result = self.execute_module(failed=True)
        self.assertEqual(result['msg'], 'Host and username are required for connection')



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2016, Eric D Helms <ericdhelms@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = '''
---
module: foreman
short_description: Manage Foreman Resources
description:
    - Allows the management of Foreman resources inside your Foreman server.
version_added: "2.3"
author:
- Eric D Helms (@ehelms)
requirements:
    - nailgun >= 0.28.0
    - python >= 2.6
    - datetime
options:
    server_url:
        description:
            - URL of Foreman server.
        required: true
    username:
        description:
            - Username on Foreman server.
        required: true
    password:
        description:
            - Password for user accessing Foreman server.
        required: true
    entity:
        description:
            - The Foreman resource that the action will be performed on (e.g. organization, host).
        required: true
    params:
        description:
            - Parameters associated to the entity resource to set or edit in dictionary format (e.g. name, description).
        required: true
'''

EXAMPLES = '''
- name: Create CI Organization
  foreman:
    username: admin
    password: admin
    server_url: https://fakeserver.com
    entity: organization
    params:
      name: My Cool New Organization
  delegate_to: localhost
'''

RETURN = '''# '''

import traceback

try:
    from nailgun import entities
    from nailgun.config import ServerConfig
    HAS_NAILGUN_PACKAGE = True
except:
    HAS_NAILGUN_PACKAGE = False

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils._text import to_native


class NailGun(object):
    def __init__(self, server, entities, module):
        self._server = server
        self._entities = entities
        self._module = module

    def find_organization(self, name, **params):
        org = self._entities.Organization(self._server, name=name, **params)
        response = org.search(set(), {'search': 'name={}'.format(name)})

        if len(response) == 1:
            return response[0]

        return None

    def organization(self, params):
        name = params['name']
        del params['name']
        org = self.find_organization(name, **params)

        if org:
            org = self._entities.Organization(self._server, name=name, id=org.id, **params)
            org.update()
        else:
            org = self._entities.Organization(self._server, name=name, **params)
            org.create()

        return True


def main():
    module = AnsibleModule(
        argument_spec=dict(
            server_url=dict(type='str', required=True),
            username=dict(type='str', required=True, no_log=True),
            password=dict(type='str', required=True, no_log=True),
            entity=dict(type='str', required=True),
            verify_ssl=dict(type='bool', default=False),
            params=dict(type='dict', required=True, no_log=True),
        ),
        supports_check_mode=True,
    )

    if not HAS_NAILGUN_PACKAGE:
        module.fail_json(msg="Missing required nailgun module (check docs or install with: pip install nailgun")

    server_url = module.params['server_url']
    username = module.params['username']
    password = module.params['password']
    entity = module.params['entity']
    params = module.params['params']
    verify_ssl = module.params['verify_ssl']

    server = ServerConfig(
        url=server_url,
        auth=(username, password),
        verify=verify_ssl
    )
    ng = NailGun(server, entities, module)

    # Lets make an connection to the server with username and password
    try:
        org = entities.Organization(server)
        org.search()
    except Exception as e:
        module.fail_json(msg="Failed to connect to Foreman server: %s " % to_native(e),
                         exception=traceback.format_exc())

    if entity == 'organization':
        ng.organization(params)
        module.exit_json(changed=True, result="%s updated" % entity)
    else:
        module.fail_json(changed=False, result="Unsupported entity supplied")


if __name__ == '__main__':
    main()
